# Буково дърво
Ветием е известна гора с много цветни дървета. Едно от най-старите е бук на име Ош Везер.

Дървото Ош Везер може да бъде представено като множество от $N$ **върха** и $N-1$ **ребра**. Върховете са номерирани с числата от $0$ до $N-1$, а ребрата са номерирани с числата от $1$ до $N-1$. Всяко ребро свързва два различни върха в дървото. По-точно, ребро $i$ ($1 \le i \lt N$) свързва връх $i$ с връх $P[i]$, където $0 \le P[i] \lt i$. Връх $P[i]$ ще наричаме **родител** на връх $i$, а връх $i$ ще наричаме **дете** на връх $P[i]$.

Всяко ребро има цвят. Съществуват $M$ на брой различни цвята номерирани с числата от $1$ до $M$. Цвета на ребро $i$ е $C[i]$. Възможно е различни ребра да имат еднакъв цвят.

Забележете, че по описаните дефиниции $i = 0$ не отговаря на никое ребро. За удобство обаче ще считаме $P[0] = -1$ и $C[0] = 0$.

За пример, нека си представим че дървото има $N = 18$ върха и $M = 3$ възможни цвята за ребрата, с $17$ ребра описани от връзките $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ и цветове на ребрата $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$. Дървото е изобразено на следната фигура:

![Tree](beechtree-tree.png "550")

Арпад е талантлив горски работник, който обича да изучава части от дървото наречени **поддървета**. За всяко $r$, такова че $0 \le r \lt N$, поддървото на връх $r$ е множеството $T(r)$ от върхове със следните свойства:
* Връх $r$ е в множеството $T(r)$.
* Ако връх $x$ е в множеството $T(r)$, то всички деца на $x$ също са в множеството $T(r)$.
* Никои други върхове не са в множеството $T(r)$.

Размерът на множеството $T(r)$ ще бележим с $|T(r)|$.

Наскоро Арпад откри сложно, но интересно свойство за някои поддървета. Откритието на Арпад изискваше доста разписване на хартия, и той подозира, че и Вие ще трябва да изминете неговия път, за да разберете свойството. Той ще ви покаже няколко примера, които може да анализирате детайлно.

Нека разгледаме даден връх $r$ и пермутация $v_0, v_1, \ldots, v_{|T(r)|-1}$ на върховете в поддървото $T(r)$.

За всяко $i$, такова че $1 \le i \lt |T(r)|$, нека $f(i)$ е броят пъти, в които цвят $C[v_i]$ се среща в следната редица от $i-1$ цвята: $C[v_1], C[v_2], \ldots, C[v_{i-1}]$.

(Забележете, че $f(1)$ е винаги $0$, понеже редицата от цветове в неговата дефиниция е празна.)

Пермутацията $v_0, v_1, \ldots, v_{|T(r)|-1}$ е **красива пермутация** тогава и само тогава, когато всички от следните свойства са верни:
* $v_0 = r$
* За всяко $i$, такова че $1 \le i \lt |T(r)|$, родителят на връх $v_i$ е връх $v_{f(i)}$.

За някое $r$, такова че $0 \le r \lt N$, казваме че поддървото $T(r)$ е **красиво поддърво** тогава и само тогава, когато съществува красива пермутация на върховете в $T(r)$. Забележете, че според тази дефиниция всяко поддърво, състоящо се от един връх, е красиво.

Да разгледаме примерното дърво по-горе. Може да се покаже, че поддървета $T(0)$ и $T(3)$ не са красиви. Поддървото $T(14)$ е красиво, тъй като се състои от единствен връх. По-долу ще покажем, че дървото $T(1)$ също е красиво.

Да разгледаме редицата от различни числа $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$. Тази редица е пермутация на върховете в $T(1)$. Фигурата по-долу изобразява тази пермутация. Числата, закачени за върховете, са индексите, на които съответните върхове се намират в пермутацията.

![Tree](beechtree-label1.png "500")

Нека верифицираме, че това е *красива пермутация*.

* $v_0 = 1$.
* $f(1) = 0$ понеже $C[v_1] = C[4] = 1$ се среща $0$ пъти в редицата $[]$.
  * Съответно, родителят на $v_1$ е $v_0$. Тоест родителят на връх $4$ е връх $1$. (Формално, $P[4] = 1$.)
* $f(2) = 0$ понеже $C[v_2] = C[5] = 2$ се среща $0$ пъти в редицата $[1]$.
  * Съответно, родителят на $v_2$ е $v_0$. Тоест родителят на връх $5$ е $1$.
* $f(3) = 1$ понеже $C[v_3] = C[12] = 1$ се среща $1$ път в редицата $[1, 2]$.
  * Съответно, родителят на $v_3$ е $v_1$. Тоест родителят на връх $12$ е $4$.
* $f(4) = 1$ понеже $C[v_4] = C[13] = 2$ се среща $1$ път в редицата $[1, 2, 1]$.
  * Съответно, родителят на $v_4$ е $v_1$. Тоест родителят на връх $13$ е $4$.
* $f(5) = 0$ понеже $C[v_5] = C[6] = 3$ се среща $0$ пъти в редицата $[1, 2, 1, 2]$.
  * Съответно, родителят на $v_5$ е $v_0$. Тоест родителят на връх $6$ е $1$.
* $f(6) = 2$ понеже $C[v_6] = C[14] = 2$ се среща $2$ пъти в редицата $[1, 2, 1, 2, 3]$.
  * Съответно, родителят на $v_6$ е $v_2$. Тоест родителят на връх $14$ е $5$.

Тъй като успяхме да намерим *красива пермутация* на върховете от $T(1)$, то поддървото $T(1)$ е *красиво поддърво*.

Вашата задача е да помогнете на Арпад да провери за всяко от поддърветата на Ош Везер дали е красиво.

## Детайли по имплементацията

Трябва да имплементирате следната функция.

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: броят върхове в дървото.
* $M$: броят възможни цветове на ребрата.
* $P$, $C$: масиви с дължина $N$, описващи ребрата в дървото.
* Тази функция трябва да връща масив $b$ с дължина $N$. За всяко $r$, такова че $0 \le r \lt N$, $b[r]$ трябва да е $1$ ако $T(r)$ е красиво поддърво, и $0$ в противен случай.
* Функцията ще бъде извикана точно веднъж за всеки тест.

## Примери

### Пример 1

Да разгледаме следното извикване:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

Дървото е изобразено на следната фигура:

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$, и $T(3)$ съдържат по точно един връх и съответно са красиви. $T(0)$ не е красиво поддърво. Затова функцията Ви трябва да върне $[0, 1, 1, 1]$.

### Пример 2

Да разгледаме следното извикване:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

Дървото е изобразено на фигурата в условието на задачата.

Функцията трябва да върне $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

<div style="page-break-after: always;"></div>

### Пример 3

Да разгледаме следното извикване:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

Дървото е изобразено на следната фигура.

![Tree](beechtree-ex3.png "400")

$T(0)$ е единственото поддърво, което не е красиво. Функцията трябва да върне $[0, 1, 1, 1, 1, 1, 1]$.


## Ограничения

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[v] \lt v$ (за всяко $v$, такова че $1 \le v \lt N$)
* $1 \le C[v] \le M$ (за всяко $v$, такова че $1 \le v \lt N$)
* $P[0] = -1$ и $C[0] = 0$

## Подзадачи

1. (9 точки) $N \le 8$ и $M \le 500$
1. (5 точки) Ребро $i$ свързва връх $i$ с връх $i-1$. Тоест, за всяко $i$, такова че $1 \le i \lt N$, $P[i] = i-1$.
1. (9 точки) Всеки връх освен връх $0$ е или свързан с връх $0$, или е свързан с връх, който е свързан с връх $0$. Това означава, че за всяко $i$, такова че $1 \le i \lt N$, или $P[i]=0$, или $P[P[i]]=0$.
1. (8 точки) За всяко $c$, такова че $1 \le c \le M$, има най-много две ребра с цвят $c$.
1. (14 точки) $N \le 200$ и $M \le 500$
1. (14 точки) $N \le 2\,000$ и $M = 2$
1. (12 точки) $N \le 2\,000$
1. (17 точки) $M = 2$
1. (12 точки) Няма допълнителни ограничения.

## Локално тестване

Локалният грейдър чете входа в следния формат:

* ред $1$: $N \; M$
* ред $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* ред $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

Нека с $b[0], \; b[1], \; \ldots$ обозначим елементите на масива върнат от `beechtree`. Локалният грейдър извежда отговора Ви на един ред в следния формат: 
* ред $1$: $b[0] \; b[1] \; \ldots$
