# El árbol de Haya

El bosque de Vétyem es una tierra muy famosa con montones de árboles coloridos.
Uno de los más altos y viejos arboles de Haya se llama Ős Vezér.

El árbol Ős Vezér puede ser modelado como un conjunto de $N$ **nodos** y $N - 1$ **aristas**.
Los nodos están enumerados de $0$ a $N - 1$, las aristas están enumeradas de $1$ a $N - 1$.
Cada arista conecta dos nodos distintos del árbol.
Específicamente, la arista $i$ ($1 \le i \lt N$) conecta al nodo $i$ con el nodo $P[i]$, donde $0 \le P[i] \lt i$. El nodo $P[i]$ es llamado **padre** del nodo $i$, y el nodo $i$ es llamado el **hijo** del nodo $P[i]$.

Cada arista tiene un color. Hay $M$ posibles colores para las aristas enumerados de $1$ a $M$.
El color de la arista $i$ es $C[i]$.
Diferentes aristas pueden tener el mismo color.

Nótese que, en las definiciones de arriba, el caso $i = 0$ no corresponde a una arista del árbol. Por conveniencia, se dice que $P[0] = -1$ y $C[0] = 0$.

Por ejemplo, supongamos que Ős Vezér tiene $N = 18$ nodos y $M = 3$ posibles colores de aristas, con $17$ aristas descritas por las conexiones $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ y los colores $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$. El árbol se muestra en la siguiente imagen:

![Tree](beechtree-tree.png "550")

Árpád es un talentoso guardabosques, y le gusta estudiar partes específicas del árbol llamadas **subárboles**.
Para todo $r$ tal que $0 \le r \lt N$, el subárbol del nodo $r$ es el conjunto $T(r)$ de nodos con las siguientes propiedades:
* El nodo $r$ pertenece a $T(r)$.
* Cuando un nodo $x$ pertenece a $T(r)$, todos los hijos de $x$ tambien pertenecen a $T(r)$.
* Ningun otro nodo pertenece a $T(r)$.

El tamaño del conjunto $T(r)$ se denota como $|T(r)|$.

Árpád recientemente descubrió una propiedad complicada pero interesante de los subárboles.
El descubrimiento de Arpád involucra demasiado uso de lapiz y papel, y sospecha que tú tambien podrias necesitarlo para entenderlo. Él tambien te mostrará varios ejemplos para que los analizes a detalle.

Supon que tenemos un valor $r$ fijo y una permutación $v_0, v_1, \ldots, v_{|T(r)|-1}$ de los nodos en el subarbol $T(r)$.

Para todo $i$ tal que $1 \le i \lt |T(r)|$, tenemos que $f(i)$ es el numero de veces que el color $C[v_i]$ aparece en la secuencia de $i-1$ colores: $C[v_1], C[v_2], \ldots, C[v_{i-1}]$.

La permutacion $v_0, v_1, \ldots, v_{|T(r)|-1}$ es **hermosa** si y solo si las siguientes propiedades se cumplen:
* $v_0 = r$.
* Para todo $i$ tal que $1 \le i \lt |T(r)|$, el padre del nodo $v_i$ es el nodo $v_{f(i)}$.

Para cualquier $r$ tal que $0 \le r \lt N$, el subárbol $T(r)$ es un **arbol hermoso** si y solo si existe una permutacion hermosa de los nodos en $T(r)$. Notese que de acuerdo a la definicion, todo subarbol que consiste de un solo nodo es hermoso.

Considera el árbol de ejemplo de arriba.
Los subárboles $T(0)$ y $T(3)$ de este árbol no son hermosos.
El subárbol $T(14)$ es hermoso, ya que contiene un solo nodo. 
Abajo, demostraremos que el subárbol $T(1)$ también es hermoso.

Considera la secuencia de enteros distintos $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$.
Esta secuencia es una permutacion de los nodos en $T(1)$. La figura de abajo ejemplifica esta permutacion.
La banderita encima de cada nodo indica el indice en el cual esos nodos aparecen en la permutacion.

![Tree](beechtree-label1.png "500")

Ahora verificaremos que esta es una *permutacion hermosa*.
* $v_0 = r = 1$.
* $f(1) = 0$ ya que $C[v_1] = C[4] = 1$ aparece $0$ veces en la secuencia $[]$.
 * Correspondientemente, el padre de $v_1$ es $v_0$. Es decir, el padre del nodo $4$ es el nodo $1$. (Mas formalmente, $P[4] = 1$.)
* $f(2) = 0$ ya que $C[v_2] = C[5] = 2$ aparece $0$ veces en la secuencia $[1]$.
 * Correspondientemente, el padre de $v_2$ es $v_0$. Es decir, el padre de $5$ es $1$.
* $f(3) = 1$ ya que $C[v_3] = C[12] = 1$ aparece $1$ vez en la secuencia $[1, 2]$.
 *  Correspondientemente, el padre de $v_3$ es $v_1$. Es decir, el padre de $12$ es $4$.
* $f(4) = 1$ ya que $C[v_4] = C[13] = 2$ aparece $1$ vez en la secuencia $[1, 2, 1]$.
 *  Correspondientemente, el padre de $v_4$ es $v_1$. Es decir, el padre de $13$ es $4$.
* $f(5) = 0$ ya que $C[v_5] = C[6] = 3$ aparece $0$ veces en la secuencia $[1, 2, 1, 2]$.
 * Correspondientemente, el padre de $v_5$ es $v_0$. Es decir, el padre de $6$ es $1$.
* $f(6) = 2$ ya que $C[v_6] = C[14] = 2$ aparece $2$ veces en la secuencia $[1, 2, 1, 2, 3]$.
 * Correspondientemente, el padre de $v_6$ es $v_2$. Es decir, el padre de $14$ es $5$.

Debido a que pudimos encontrar una *permutacion hermosa* de los nodos en $T(1)$, el subarbol $T(1)$ es un *arbol hermoso*.

Tu tarea es ayudar a Árpád a decidir, para todo subarbol de Ős Vezér, si es que es hermoso.

## Detalles de Implementación

Debes implementar la siguiente función.

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: el número de nodos en el árbol.
* $M$: el número de colores de arista.
* $P$, $C$: arreglos de tamaño $N$ describiendo las aristas del árbol.
* Esta función debe devolver un arreglo $b$ de tamaño $N$. 
  Para todo $r$ tal que $0 \le r \lt N$, $b[r]$ debe ser $1$ si es que $T(r)$ es hermoso, y $0$ en caso contrario.
* Esta función es llamada exactamente una vez por cada caso de prueba.

## Ejemplos

### Ejemplo 1

Considera la siguiente llamada:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

El árbol se muestra en la imagen:

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$, y $T(3)$, cada uno contiene un solo nodo y por lo tanto son hermosos.
$T(0)$ no es hermoso.
Por tanto, la función debe retornar $[0, 1, 1, 1]$.

### Ejemplo 2

Considera la siguiente llamada:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

Este ejemplo es mostrado en la descripción del problema, la cual esta arriba.

La función debe retornar $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

### Ejemplo 3

Considera la siguiente llamada:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

Este ejemplo se muestra en la imagen.

![Tree](beechtree-ex3.png "400")

$T(0)$ es el único subárbol que no es hermoso.
La función debe retornar $[0, 1, 1, 1, 1, 1, 1]$.


## Límites

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[i] \lt i$ (para todo $i$ tal que $1 \le i \lt N$)
* $1 \le C[i] \le M$ (para todo $i$ tal que $1 \le i \lt N$)
* $P[0] = -1$ y $C[0] = 0$

## Subtareas

1. (9 puntos) $N \le 8$ y $M \le 500$
1. (5 points) La arista $i$ conecta el nodo $i$ con el nodo $i-1$. Es decir, para todo $i$ tal que $1 \le i \lt N$, $P[i] = i-1$.
1. (9 puntos) Todo nodo exceptuando el nodo $0$ está conectado al nodo $0$, o está conectado a un nodo el cual está conectado al nodo $0$.
  Es decir, para todo $i$ tal que $1 \le i \lt N$, se cumplirá que $P[i]=0$ o $P[P[i]]=0$.
1. (8 puntos) Para todo $c$ tal que $1 \le c \le M$, hay a lo mucho dos aristas de color $c$.
1. (14 puntos) $N \le 200$ y $M \le 500$
1. (14 puntos) $N \le 2\,000$ y $M = 2$
1. (12 puntos) $N \le 2\,000$
1. (17 puntos) $M = 2$
1. (12 puntos) Sin restricciones adicionales.

## Evaluador de ejemplo

El evaluador de ejemplo lee la entrada siguiendo el formato:

* línea $1$: $N \; M$
* línea $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* línea $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

Sea que $b[0], \; b[1], \; \ldots$ denote a los elementos del arreglo retornado por la función `beechtree`.
El evaluador de ejemplo imprimirá tu respuesta en una sola línea, con el siguiente formato:
* línea $1$: $b[0] \; b[1] \; \ldots$
