# Árbol del haya

Vétyem Woods es un bosque famoso con muchos árboles coloridos. Una de las hayas más antiguas y altas se llama Ős Vezér.

El árbol Os Vezér se puede modelar como un conjunto de $N$ **nodos** y $N-1$ **aristas**.
Los nodos están numerados desde $0$ hasta $N-1$ y las aristas están numerados desde $1$ hasta $N-1$.
Cada arista conecta dos nodos distintos del árbol. Específicamente, arista $v$ ($1 \le v \lt N$) conecta el nodo $v$ hasta el nodo $P[v]$, donde $0 \le P[v] \lt v$.

Cada arista tiene un color. Hay $M$ posibles colores de arista numerados desde $1$ to $M$.
El color del arista $v$ e $C[v]$.
Diferentes aristas pueden tener el mismo color.

Tenga en cuenta que en las definiciones anteriores, el caso $v = 0$ no corresponde a un arista del árbol. Por conveniencia, dejamos $P[0] = -1$ y $C[0] = 0$.

Por ejemplo, supongamos que Ős Vezér tiene $N = 18$ nodos y $M = 3$ posibles colores de arista, con $17$ aristas descritos por conexiones $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ y coloes $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$. El árbol se muestra en la siguiente figura:

![Tree](beechtree-tree.png "550")

Árpád es un ingeniero forestal talentoso al que le gusta estudiar partes específicas del árbol llamadas **subárboles**.
Para cada $r$ tal que $0 \le r \lt N$, el subárbol del nodo $r$ (denotado por $T(r)$) es un conjunto de nodos.
Un nodo $s$ es miembro de $T(r)$ si y solo si:
* $s$ = $r$, o
* $s \gt r$, y el nodo $P[s]$ es un miembro de $T(r)$.

El tamaño del conjunto $T(r)$ se denota como $|T(r)|$.

Árpád descubrió una interesante propiedad del subárbol. Para cada $r$ tal que  $0 \le r \lt N$, el subárbol $T(r)$ es **complicado** si y sólo si existe una secuencia de números enteros *distintos*  $[v_0, v_1, \ldots, v_{|T(r)|-1}]$ tal que:
* Para cada $i$ tal que $0 \le i \lt |T(r)|$, nodo $v_i$ es miembro de $T(r)$.
* $v_0 = r$.
* Para cada $i$ tal que $1 \le i \lt |T(r)|$, $P[v_i] = v_{f(i)}$, where $f(i)$ es definido como el número de veces que el color $C[v_i]$ aparece en la secuencia $[C[v_1], C[v_2], \ldots, C[v_{i-1}]]$.

Tenga en cuenta que según la definición:
* Cada subárbol que contiene un solo nodo es complicado.
* Para cualquier subárbol que contenga dos o más nodos, $f(1) = 0$ porque la secuencia de colores en su definición está vacía.

Considere el árbol de ejemplo anterior. 
Los subárboles $T(0)$ and $T(3)$ de este árbol no son complicados.
El subárbol $T(14)$ es complicado, ya que contiene un solo nodo. 
A continuación, mostraremos que el subárbol $T(1)$ también es complicado.

Considere la secuencia de números enteros distintos. $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$.
Esta secuencia se representa en la siguiente figura. El índice de cada nodo en esta secuencia se muestra mediante el número en la etiqueta adjunta al nodo.

![Tree](beechtree-label1.png "500")

Claramente, la secuencia anterior es una permutación de los nodos en $T(1)$. Ahora verificaremos que se trata de una *hermosa permutación *.

* $v_0 = 1$.
* $f(1) = 0$ ya que $C[v_1] = C[4] = 1$ aparece $0$ veces en la secuencia $[]$.
  * En consecuencia, el padre de $v_1$ es $v_0$. Es decir, el padre del nodo $4$ es el nodo $1$. (Formalmente, $P[4] = 1$.)
* $f(2) = 0$ ya que $C[v_2] = C[5] = 2$ aparece $0$ veces en la secuencia $[1]$.
  * En consecuencia, el padre de $v_2$ es $v_0$. Es decir, el padre de $5$ es $1$.
* $f(3) = 1$ ya que $C[v_3] = C[12] = 1$ aparece $1$ vez en la secuencia $[1, 2]$.
  * En consecuencia, el padre de $v_3$ es $v_1$. Es decir, el padre de $12$ es $4$.
* $f(4) = 1$ ya que $C[v_4] = C[13] = 2$ aparece $1$ vez en la secuencia $[1, 2, 1]$.
  * En consecuencia, el padre de $v_4$ es $v_1$. Es decir, el padre de $13$ es $4$.
* $f(5) = 0$ ya que $C[v_5] = C[6] = 3$ aparece $0$ veces en la secuencia $[1, 2, 1, 2]$.
  * En consecuencia, el padre de $v_5$ es $v_0$. Es decir, el padre de $6$ es $1$.
* $f(6) = 2$ ya que $C[v_6] = C[14] = 2$ aparece $2$ veces en la secuencia $[1, 2, 1, 2, 3]$.
  * En consecuencia, el padre de $v_6$ es $v_2$. Es decir, el padre de $14$ es $5$.

Como pudimos encontrar una * hermosa permutación * de los nodos en $T(1)$, el subárbol $T(1)$ es de hecho un * hermoso . subárbol*. 

## Detalles de la implementación

Debe implementar el siguiente procedimiento.

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: el número de nodos en el árbol.
* $M$: el número de  posibles arista de colores.
* $P$, $C$: matrices de longitud $N$ describiendo las aristas del árbol.
* Este procedimiento debe devolver una matriz. $b$ de longitud $N$. 
  Para cada $r$ tal que $0 \le r \lt N$, $b[r]$ debiera ser $1$ if $T(r)$ es complicado, y $0$ en caso contrario.
* Este procedimiento se llama exactamente una vez para cada caso de prueba..

## Ejemplos

### Ejemplo 1

Considere la siguiente llamada:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

El árbol se muestra en la siguiente figura:

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$, y $T(3)$ cada uno contiene un solo nodo y, por lo tanto, son complicados.
$T(0)$no es complicado. 
Por lo tanto, el procedimiento debería volver $[0, 1, 1, 1]$.

### Ejemplo 2

Considere la siguiente llamada:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

Este ejemplo se ilustra en la descripción de la tarea anterior.

El procedimiento debe retornar $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

### Ejemplo 3

Considere la siguiente llamada:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

Este ejemplo se ilustra en la siguiente figura.

![Tree](beechtree-ex3.png "400")

$T(0)$ es el único subárbol que no es complicado. 
El procedimiento debe retornar $[0, 1, 1, 1, 1, 1, 1]$.


## Restricciones

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[v] \lt v$ (para  cada $v$ tal que $1 \le v \lt N$)
* $1 \le C[v] \le M$ (para  cada $v$ tal que $1 \le v \lt N$)
* $P[0] = -1$ y $C[0] = 0$

## Subtareas

1. (9 puntos) $N \le 8$ y $M \le 500$
1. (5 puntos) El borde $i$ conecta el nodo $i$ con el nodo $i-1$. Es decir, para cada $i$ tal que $1 \le i \lt N$, $P[i] = i-1$.
1. (9 puntos) Cada nodo que no sea el nodo $0$ está conectado al nodo $0$ o está conectado a un nodo que está conectado al nodo $0$.
  Es decir, para cada $ v i $ tal que $1 \le v i \lt N$, $P[ v i ]=0$ o $P[P[ v i ]]=0$.
1. (8 puntos) Para cada $c$ tal que $1 \le c \le M$, hay como máximo dos aristas de color $c$.
1. (14 puntos) $N \le 200$ y $M \le 500$
1. (14 puntos) $N \le 2\,000$ y $M = 2$
1. (12 puntos) $N\le 2\,000$
1. (17 puntos) $M = 2$
1. (12 puntos) Sin restricciones adicionales.

## Calificador de ejemplo Grader

El calificador de ejemplo lee la entrada en el siguiente formato:

* línea $1$: $N \; M$
* línea $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* línea $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

Sea $b[0], \; b[1], \; \ldots$ denota los elementos de la matriz devuelta por `beechtree`.
El calificador de ejemplo imprime su respuesta en una sola línea, en el siguiente formato:
* línea $1$: $b[0] \; b[1] \; \ldots$
