# Hêtre

Dans la célèbre forêt de Vétyem poussent de nombreux arbres colorés.
L'un des plus vieux et plus haut de ces arbres est appelé Ős Vezér.

L'arbre Ős Vezér peut être modélisé par un ensemble de $N$ **sommets** et $N-1$ **arêtes**.
Les sommets sont numérotés de $0$ à $N-1$ et les arêtes sont numérotées de $1$ à $N-1$.
Chaque arête relie deux sommets différents de l'arbre.
Plus précisément, l'arête $i$ ($1 \le i \lt N$) relie le sommet $i$ au sommet $P[i]$, où $0 \le P[i] \lt i$. Le sommet $P[i]$ est le **parent** du sommet $i$, et le sommet $i$ est un **fils** du sommet $P[i]$.

Chaque arête est d'une certaine couleur. Il y a $M$ couleurs possibles, numérotées de $1$ à $M$.
La couleur de l'arête $i$ est $C[i]$.
Des arêtes différentes peuvent être de la même couleur.

Notez que dans les définitions ci-dessus, le cas $i = 0$ ne correspond à aucune arête de l'arbre. Par praticité, on fixera $P[0] = -1$ et $C[0] = 0$.

Par exemple, supposons qu'Ős Vezér a $N = 18$ sommets et $M = 3$ couleurs possibles pour les $17$ arêtes décrites par les connexions $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ et les couleurs $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$. L'arbre est représenté dans la figure suivante :

![Tree](beechtree-tree.png "550")

Árpád est un forestier talentueux, qui aime étudier des parties spécifiques de l'arbre appelées **sous-arbres**.
Pour chaque $r$ tel que $0 \le r \lt N$, le sous-arbre de sommet $r$ est l'ensemble $T(r)$ de sommet qui vérifie les propriétés suivantes :
* Le sommet $r$ fait partie de $T(r)$.
* Lorsqu'un sommet $x$ fait partie de $T(r)$, tous les fils de $x$ font aussi partie de $T(r)$.
* Aucun autre sommet ne fait partie de $T(r)$.

La taille de l'ensemble $T(r)$ est noté $|T(r)|$.

Árpád a récemment découvert une propriété compliquée mais intéressante des sous-arbres. La découverte d'Árpád a nécessité beaucoup de réflexion avec un papier et un crayon, et il suspecte que vous devriez faire de même pour la comprendre en détails. Il va aussi vous montrer plusieurs exemples que vous pourrez analyser en détails.

Supposons avoir choisi $r$ et une permutation $v_0, v_1, \ldots, v_{|T(r)|-1}$ des sommets du sous-arbre $T(r)$.

Pour chaque $i$ tel que $1 \le i \lt |T(r)|$, soit $f(i)$ le nombre de fois que la couleur $C[v_i]$ apparaît dans l'ensemble de $i-1$ couleurs suivant : $C[v_1], C[v_2], \ldots, C[v_{i-1}]$. 

(Notez que $f(1)$ est toujours $0$ car l'ensemble des couleurs qui le défini est vide.)

La permutation $v_0, v_1, \ldots, v_{|T(r)|-1}$ est une **permutation éblouissante** si et seulement si toutes les propriétés suivantes sont vérifiées :
* $v_0 = r$.
* Pour chaque $i$ tel que $1 \le i \lt |T(r)|$, le parent du sommet $v_i$ est le sommet $v_{f(i)}$.

Pour chaque $r$ tel que $0 \le r \lt N$, le sous-arbre $T(r)$ est un **sous-arbre éblouissant** si et seulement s'il existe une permutation éblouissante des sommets de $T(r)$. Notez que d'après cette définition, tout sous-arbre constitué d'un seul sommet est éblouissant.

Considérons l'exemple d'arbre précédent.
On peut démontrer que les sous-arbres $T(0)$ et $T(3)$ de cet arbre ne sont pas éblouissants.
Le sous-arbre $T(14)$ est éblouissant, puisqu'il ne contient qu'un seul sommet.
Dans la suite, nous allons montrer que le sous-arbre $T(1)$ est aussi éblouissant.

Considérons la suite d'entiers distincts $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$. Cette suite est une permutation des sommets de $T(1)$. La figure ci-dessous représente cette permutation. Les étiquettes attachées aux sommets sont les indices auxquels ces sommets apparaîssent dans la permutation.

![Tree](beechtree-label1.png "500")

Nous allons maintenant vérifier qu'il s'agit d'une *permutation éblouissante*.

* $v_0 = 1$.
* $f(1) = 0$ puisque $C[v_1] = C[4] = 1$ apparaît $0$ fois dans la suite $[]$.
  * D'autre part, le parent de $v_1$ est $v_0$. C'est à dire que le parent du sommet $4$ est le sommet $1$. (Formellement, $P[4] = 1$.)
* $f(2) = 0$ puisque $C[v_2] = C[5] = 2$ apparaît $0$ fois dans la suite $[1]$.
  * D'autre part, le parent de $v_2$ est $v_0$. C'est à dire que le parent du sommet $5$ est le sommet $1$.
* $f(3) = 1$ puisque $C[v_3] = C[12] = 1$ apparaît $1$ fois dans la suite $[1, 2]$.
  * D'autre part, le parent de $v_3$ est $v_1$. C'est à dire que le parent du sommet $12$ est le sommet $4$.
* $f(4) = 1$ puisque $C[v_4] = C[13] = 2$ apparaît $1$ fois dans la suite $[1, 2, 1]$.
  * D'autre part, le parent de $v_4$ est $v_1$. C'est à dire que le parent du sommet $13$ est le sommet $4$.
* $f(5) = 0$ puisque $C[v_5] = C[6] = 3$ apparaît $0$ fois dans la suite $[1, 2, 1, 2]$.
  * D'autre part, le parent de $v_5$ est $v_0$. C'est à dire que le parent du sommet $6$ est le sommet $1$.
* $f(6) = 2$ puisque $C[v_6] = C[14] = 2$ apparaît $2$ fois dans la suite $[1, 2, 1, 2, 3]$.
  * D'autre part, le parent de $v_6$ est $v_2$. C'est à dire que le parent du sommet $14$ est le sommet $5$.

Comme nous avons pu trouver une *permutation éblouissante* des sommets de $T(1)$, le sous-arbre $T(1)$ est un *sous-arbre éblouissant*.

Votre tâche est d'aider Árpád à déterminer quels sous-arbres d'Ős Vezér sont éblouissants.

## Détails d'implémentation

Vous devez implémenter la fonction suivante :

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: le nombre de sommets dans l'arbre.
* $M$: le nombre de couleurs possibles pour les arêtes.
* $P$, $C$: des tableaux de longueur $N$ décrivant les arêtes de l'arbre.
* Cette fonction doit renvoyer un tableau $b$ de longueur $N$. 
  Pour chaque $r$ tel que $0 \le r \lt N$, $b[r]$ doit valoir $1$ si $T(r)$ est éblouissant, et $0$ sinon.
* Cette fonction est appelée exactement une fois par test.

## Exemples

### Exemple 1

Considérons l'appel suivant :

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

L'arbre est représenté dans la figure suivante :

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$, et $T(3)$ sont tous constitués d'un seul sommet et sont donc éblouissants.
$T(0)$ n'est pas éblouissant.
Ainsi, la fonction doit renvoyer $[0, 1, 1, 1]$.

### Exemple 2

Considérons l'appel suivant :

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

Cet exemple est illustré précédemment dans l'énoncé du sujet.

La fonction doit renvoyer $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

### Exemple 3

Considérons l'appel suivant :

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

Cet exemple est illustré dans la figure suivante.

![Tree](beechtree-ex3.png "400")

$T(0)$ est le seul sous-arbre qui n'est pas éblouissant.
La fonction doit renvoyer $[0, 1, 1, 1, 1, 1, 1]$.


## Contraintes

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[i] \lt i$ (pour chaque $i$ tel que $1 \le i \lt N$)
* $1 \le C[i] \le M$ (pour chaque $i$ tel que $1 \le i \lt N$)
* $P[0] = -1$ et $C[0] = 0$

## Sous-tâches

1. (9 points) $N \le 8$ et $M \le 500$
1. (5 points) L'arête $i$ relie le sommet $i$ au sommet $i-1$. Autrement dit, pour chaque $i$ tel que $1 \le i \lt N$, $P[i] = i-1$.
1. (9 points) Chaque sommet autre que le sommet $0$ est soit relié au sommet $0$, soit est relié à un sommet qui est relié au sommet $0$.
  Autrement dit, pour chaque $i$ tel que $1 \le i \lt N$, on a $P[i]=0$ ou $P[P[i]]=0$.
1. (8 points) Pour chaque $c$ tel que $1 \le c \le M$, il y a au plus deux arêtes de couleur $c$.
1. (14 points) $N \le 200$ et $M \le 500$
1. (14 points) $N \le 2\,000$ et $M = 2$
1. (12 points) $N \le 2\,000$
1. (17 points) $M = 2$
1. (12 points) Aucune contrainte supplémentaire.

## Évaluateur d'exemple (grader)

L'évaluateur lit l'entrée au format suivant :

* ligne $1$: $N \; M$
* ligne $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* ligne $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

Soient $b[0], \; b[1], \; \ldots$ les éléments du tableau renvoyé par `beechtree`.
L'évaluateur affiche votre réponse sur une unique ligne au format suivant :
* ligne $1$: $b[0] \; b[1] \; \ldots$