# Bükkfa (Beech Tree)

A Vétyemi bükkös egy híres erdő, sok színes fával.
Az egyik legöregebb és legmagasabb bükkfát Ős Vezérnek hívják.

Az Ős Vezér fa modellezhető $N$ **csomópontból** és $N-1$ **élből** álló halmazként.
A csomópontokat $0$-tól $N-1$-ig, az éleket pedig $1$-tól $N-1$-ig számozzuk.
Minden él a fa két különböző csomópontját köti össze.
Vagyis  a $v$ él ($1 \le v \lt N$) a $v$ csomópontot a $P[v]$ csomóponttal köti össze, ahol $0 \le P[v] \lt v$.

Minden élnek van színe. Az $M$ lehetséges élszínt $1$-tól $M$-ig számozzuk.
Az $v$. él színe $C[v]$.
Különböző éleknek lehet ugyanaz a színe.

Vegyük észre, hogy a fenti definíciókban a $v = 0$ eset nem faél. Az egyszerűség kedvéért legyen $P[0] = -1$ és $C[0] = 0$.

Például tegyük fel, hogy Ős Vezérnek $N = 18$ csomópontja és $M = 3$ lehetséges élszíne van. A $17$ darab élt a  $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ kapcsolatok és a $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$ színek írják le. A fa a következő ábrán látható:

![Fa](beechtree-tree.png "550")

Árpád egy tehetséges erdész, aki szívesen tanulmányozza a fa egyes részeit, az úgynevezett **részfákat**.

Minden olyan $r$ esetében, ahol $0 \le r \lt N$, az $r$ csomópont részfája a következő tulajdonságokkal rendelkező csomópontok $T(r)$ halmaza:
* Az $r$ csomópont a $T(r)$ halmazhoz tartozik.
* Ha egy $x$ csomópont $T(r)$-hez tartozik, akkor $x$ minden gyermeke szintén $T(r)$-hez tartozik.
* Nincs más csomópont, amelyik a $T(r)$-hoz tartozik.

A $T(r)$ halmaz méretét $|T(r)|$-val jelöljük.

Árpád nemrég felfedezett egy bonyolult, de érdekes részfa tulajdonságot. Árpád felfedezéséhez rengeteget játszott tollal és papírral, és gyanítja, hogy neked is ugyanezt kell tenned, hogy megértsd. Több példát is mutat majd, amit aztán részletesen elemezhetsz.

Tegyük fel, hogy $r$ egy rögzített  csomópont és  és  $v_0, v_1, \ldots, v_{|T(r)|-1}$ sorozat a $T(r)$ részfa csomópontjainak egy  permutációja..

Minden $i$-re ($1 \le i \lt |T(r)|$), legyen $f(i)$ a $C[v_i]$ szín megjelenéseinek száma a következő, $i-1$ színt tartalmazó sorozatban:  $[C[v_1], C[v_2], \ldots, C[v_{i-1}]]$ .

(Megjegyzés: az $f(1)$ mindig $0$, mert a definícióban szereplő színsorozat üres.)

A $v_0, v_1, \ldots, v_{|T(r)|-1}$ permutáció akkor és csak akkor **szépséges permutáció**, ha a következő tulajdonságok mindegyike teljesül:
* $v_0 = r$.
* Minden olyan $i$ esetében, ahol $1 \le i \lt |T(r)|$, a $v_i$ csúcspont őse a $v_{f(i)}$ csúcspont.

Bármely $r$ esetén, ahol $0 \le r \lt N$, a $T(r)$ részfa akkor és csak akkor **szépséges részfa**, ha létezik a $T(r)$ csomópontjainak szépséges permutációja. Megjegyezzük, hogy a definíció szerint minden olyan részfa szépséges, amely egyetlen csomópontból áll.

Tekintsük a fenti példafát.
Bizonyítható, hogy a fa $T(0)$ és $T(3)$ részfái nem szépségesek.
A $T(14)$ részfa viszont szépséges, mivel egyetlen csomópontot tartalmaz.
Az alábbiakban megmutatjuk, hogy a $T(1)$ részfa is szépséges.

Tekintsük a különböző egész számok  $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$ sorozatát.
Ez a sorozat a $T(1)$ csomópontjainak egy permutációja. Az alábbi ábra ezt a permutációt ábrázolja. A csomópontokhoz tartozó címkék azok az indexek, amelyeken a csomópontok a permutációban szerepelnek.

![Tree](beechtree-label1.png "500")

Nyilvánvaló, hogy a fenti sorozat a $T(1)$ csomópontjainak permutációja. Most bizonyítjuk, hogy ez egy *szépséges* permutáció.

* $v_0 = 1$.
* $f(1) = 0$, mivel $C[v_1] = C[4] = 1$ érték, ami $0$-szor fordul elő a $[]$ sorozatban.
  * Ennek megfelelően $v_1$ őse $v_0$ (azaz $4$ őse $1$). Formálisan: $P[4] = 1$.
* $f(2) = 0$, mivel $C[v_2] = C[5] = 2$ érték, ami $0$-szor fordul elő a $[1]$ sorozatban.
  * Ennek megfelelően $v_2$ őse $v_0$ (azaz $5$ őse $1$).
* $f(3) = 1$, mivel $C[v_3] = C[12] = 1$ érték, ami $1$ alkalommal fordul elő a $[1, 2]$ sorozatban.
  * Ennek megfelelően $v_3$ őse $v_1$ (azaz $12$ őse $4$).
* $f(4) = 1$, mivel $C[v_4] = C[13] = 2$ érték, ami $1$ alkalommal fordul elő a $[1, 2, 1]$ sorozatban.
  * Ennek megfelelően $v_4$ őse $v_1$ (azaz $13$ őse $4$).
* $f(5) = 0$, mivel $C[v_5] = C[6] = 3$ érték, ami $0$ alkalommal fordul elő a $[1, 2, 1, 1, 2]$ sorozatban.
  * Ennek megfelelően $v_5$ őse $v_0$ (azaz $6$ őse $1$).
* $f(6) = 2$, mivel $C[v_6] = C[14] = 2$ érték, ami $2$ alkalommal fordul elő a $[1, 2, 1, 2, 2, 3]$ sorozatban.
  * Ennek megfelelően $v_6$ őse $v_2$. (Vagyis a $14$ őse az $5$.)

Mivel megtaláltuk a $T(1)$ csomópontjainak egy *szépséges permutációját*, a $T(1)$ részfa valóban *szépséges részfa*.

A feladatod, hogy segíts Árpádnak eldönteni Ős Vezér minden egyes részfájáról, hogy az szépséges-e.

## Megvalósítás

A következő függvényt kell megvalósítanod:

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: a csomópontok száma a fában.
* $M$: a lehetséges élszínek száma.
* $P$, $C$: $N$ elemű tömb, ami a fa éleit írja le.
* A függvény visszatérési értéke az $N$ elemű  $b$ tömb. 
  Minden $r$-re ( ahol $0 \le r \lt N$), $b[r]$ legyen $1$, ha $T(r)$ szépséges, és $0$ egyébként.
* Ez a függvény minden egyes tesztesetben pontosan egyszer kerül meghívásra.

## Példák

### Példa 1

Nézzük  a következő függvényhívást:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

A fa ábrája:

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$, és $T(3)$ mindegyike egyetlen csomópontot tartalmaz, tehát szépséges.
$T(0)$ nem szépséges.
A függvény visszatérési értéke:  $[0, 1, 1, 1]$.

### Példa 2

Nézzük  a következő függvényhívást:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

A fa ábrája feljebb, a feladatleírásban található.

A függvény visszatérési értéke: $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

### Példa 3

Nézzük  a következő függvényhívást:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

A fa ábrája:

![Tree](beechtree-ex3.png "400")

$T(0)$ az egyetlen részfa, ami nem szépséges.
A függvény visszatérési értéke: $[0, 1, 1, 1, 1, 1, 1]$.


## Feltételek

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[i] \lt i$ ( minden $i$-re, ahol $1 \le i \lt N$)
* $1 \le C[i] \le M$ ( minden $i$-re, ahol $1 \le i \lt N$)
* $P[0] = -1$ és $C[0] = 0$

## Részfeladatok

1. (9 pont) $N \le 8$ és $M \le 500$
1. (5 pont) Az $i$. él a $i$. csomópontot köti a $i-1$-hez. Vagyis minden olyan $i$ esetén, ahol $1 \le i \lt N$, $P[i] = i-1$.
1. (9 pont) A $0$ csomóponton kívül minden csomópont vagy kapcsolódik a $0$ csomóponthoz, vagy kapcsolódik egy olyan csomóponthoz, amely kapcsolódik a $0$ csomóponthoz. Vagyis minden olyan $i$ esetében, ahol $1 \le i \lt N$, vagy $P[i]=0$ vagy $P[P[i]]=0$.
1. (8 pont) Minden olyan $c$ esetében, hogy $1 \le c \le M$, legfeljebb két $c$ színű él van.
1. (14 pont) $N \le 200$ and $M \le 500$
1. (14 pont) $N \le 2\,000$ and $M = 2$
1. (12 pont) $N \le 2\,000$
1. (17 pont) $M = 2$
1. (12 pont) Nincs további megkötés.

## Mintaértékelő

A mintaértékelő a következő formátumban olvassa be a bemenetet:

* $1$. sor: $N \; M$
* $2$. sor: $P[0] \; P[1] \; \ldots \; P[N-1]$
* $3$. sor: $C[0] \; C[1] \; \ldots \; C[N-1]$

Legyen $b[0], \; b[1], \; \ldots$ a `beechtree` által visszaadott tömb elemei.
A mintaértékelő egyetlen sorban, a következő formátumban írja ki a választ:
* $1$. sor: $b[0] \; b[1] \; \ldots$
