# Beech Tree

Ветьемский лес &mdash; знаменитый лесной массив с множеством разноцветных деревьев. Одно из старейших и самых высоких буковых деревьев называется Ос Везер.

Дерево Ос Везер может быть описано множествами из $N$ **вершин** и $N-1$ **ребер**. 
Вершины пронумерованы от $0$ до $N-1$ и ребра пронумерованы от $1$ до $N-1$.
Каждое ребро соединяет две различных вершины дерева. Ребро $i$ ($1 \le i \lt N$) соединяет вершину $i$ с вершиной $P[i]$, где $0 \le P[i] \lt i$. Вершина $P[i]$ называется **родителем** вершины $i$ и вершина $i$ называется **сыном** вершины $P[i]$.

Каждое ребро имеет цвет. Всего есть $M$ различных цветов, они пронумерованы от $1$ до $M$.
Цвет ребра $i$ это $C[i]$. Различные ребра могут иметь одинаковый цвет.

Обратите внимание, что в определении выше $i = 0$ не соответствует ребру дерева. Для удобства будем считать что $P[0] = -1$ и $C[0] = 0$.

Например, предположим, что Ос Везер содержит $N = 18$ вершин и есть $M = 3$ возможных цвета ребер. Ребра описываются последовательностью $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$, их цвета последовательностью $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$. Ниже приведено изображение дерева:

![Tree](beechtree-tree.png "550")

Арпад это талантливый лесник, который изучает отдельные части дерева, называемые **поддеревьями**. Для всех $r$, таких что $0 \le r \lt N$, поддерево вершины $r$ это множество вершин $T(r)$, обладающее следующими свойствами:
* Вершина $r$ лежит в $T(r)$.
* Если вершина $x$ лежит в $T(r)$, все сыновья $x$ тоже лежат в $T(r)$.
* Никакие другие вершины не лежат в $T(r)$.

Размер множества $T(r)$ обозначим как $|T(r)|$.

Арпад недавно открыл сложное, но интересное свойство поддеревьев. Открытие Арпада потребовало от него много работы с ручкой и бумагой, и он подозревает, что вам, возможно, придется сделать то же самое, чтобы понять его. Он покажет вам несколько примеров, которые вы можете детально проанализировать.

Пусть мы зафиксировали $r$ и перестановку $v_0, v_1, \ldots, v_{|T(r)|-1}$ вершин $T(r)$.

Для всех $i$ таких что $1 \le i \lt |T(r)|$, обозначим за $f(i)$ количество раз, которое цвет $C[v_i]$ встречается в следующей последовательности из $i-1$ цветов: $C[v_1], C[v_2], \ldots, C[v_{i-1}]$.

(Обратите внимание, что $f(1)$ всегда равно $0$, потому что последовательность цветов в определении пустая.)

Перестановка $v_0, v_1, \ldots, v_{|T(r)|-1}$ **красивая** тогда и только тогда, когда следующие свойства выполнены:
* $v_0 = r$.
* Для всех $i$ таких что $1 \le i \lt |T(r)|$, родитель вершины $v_i$ это вершина $v_{f(i)}$.

Для всех $r$ таких что $0 \le r \lt N$, поддерево $T(r)$ **красивое** тогда и только тогда, когда существует красивая перестановка вершин поддерева $T(r)$. Обратите внимание, что по определению каждое поддерево, которое состоит из единственной вершины красивое.

Рассмотрим пример дерева выше. Поддеревья $T(0)$ и $T(3)$ этого дерева некрасивые. Поддерево $T(14)$ красивое, потому что оно состоит из единственной вершины. Ниже мы показываем, что поддерево $T(1)$ также красивое.

Рассмотрим последовательность различных целых чисел $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$. Эта последовательность это перестановка вершин поддерева $T(1)$. Эта перестановка показана на изображении снизу. Номер каждой вершины в этой перестановке это число, записанное на табличке рядом с вершиной.

![Tree](beechtree-label1.png "500")

Данная последовательность это перестановка вершин в $T(1)$. Давайте проверим, что она *красивая*.

* $v_0 = 1$.
* $f(1) = 0$, потому что $C[v_1] = C[4] = 1$ встречается $0$ раз в последовательности $[]$.
* Соответственно родитель вершины $v_1$ это $v_0$. А именно, родитель вершины $4$ это $1$. (Формально, $P[4] = 1$.)
* $f(2) = 0$, потому что $C[v_2] = C[5] = 2$ встречается $0$ раз в последовательности $[1]$.
* Соответственно родитель вершины $v_2$ это $v_0$. А именно, родитель вершины $5$ это $1$.
* $f(3) = 1$, потому что $C[v_3] = C[12] = 1$ встречается $1$ раз в последовательности $[1, 2]$.
* Соответственно родитель вершины $v_3$ это $v_1$. А именно, родитель вершины $12$ это $4$.
* $f(4) = 1$, потому что $C[v_4] = C[13] = 2$ встречается $1$ раз в последовательности $[1, 2, 1]$.
* Соответственно родитель вершины $v_4$ это $v_1$. А именно, родитель вершины $13$ это $4$.
* $f(5) = 0$, потому что $C[v_5] = C[6] = 3$ встречается $0$ раз в последовательности $[1, 2, 1, 2]$.
* Соответственно родитель вершины $v_5$ это $v_0$. А именно, родитель вершины $6$ это $1$.
* $f(6) = 2$, потому что $C[v_6] = C[14] = 2$ встречается $2$ раза в последовательности $[1, 2, 1, 2, 3]$.
* Соответственно родитель вершины $v_6$ это $v_2$. А именно, родитель вершины $14$ это $5$.

Таким образом, мы нашли красивую перестановку вершин поддерева $T(1)$, поэтому поддерево $T(1)$ красивое.

Ваша задача помочь Арпаду для каждого поддерева Ос Везера определить, является ли оно красивым.

## Implementation Details

Вы должны реализовать следующую функцию.

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: количество вершин в дереве.
* $M$: количество возможных цветов ребер.
* $P$, $C$: массивы длины $N$, описывающие ребра дерева.
* Эта функция должна вернуть массив $b$ длины $N$. 
  Для всех $r$, таких что $0 \le r \lt N$, $b[r]$ должно быть равно $1$, если $T(r)$ красивое поддерево и $0$, иначе.
* Эта функция вызывается ровно один раз для каждого теста.

## Examples

### Example 1

Рассмотрим следующий вызов:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

Это дерево изображено ниже:

![Tree](beechtree-ex1.png "350")

Поддеревья $T(1)$, $T(2)$ и $T(3)$ состоят из единственной вершины и поэтому являются красивыми. Поддерево $T(0)$ некрасивое. Таким образом, функция должна вернуть $[0, 1, 1, 1]$.

### Example 2

Рассмотрим следующий вызов:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

Это дерево изображено выше в условии задачи.

Функция должна вернуть $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

### Example 3

Рассмотрим следующий вызов:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

Это дерево изображено ниже:

![Tree](beechtree-ex3.png "400")

$T(0)$ это единственное некрасивое поддерево.
Функция должна вернуть $[0, 1, 1, 1, 1, 1, 1]$.


## Constraints

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[i] \lt i$ (для всех $i$ таких что $1 \le i \lt N$)
* $1 \le C[i] \le M$ (для всех $i$ таких что $1 \le i \lt N$)
* $P[0] = -1$ и $C[0] = 0$

## Subtasks

1. (9 баллов) $N \le 8$ и $M \le 500$
1. (5 баллов) Ребро $i$ соединяет вершину $i$ с вершиной $i-1$. То есть для всех $i$ таких что $1 \le i \lt N$, $P[i] = i-1$.
1. (9 баллов) Каждая вершина, кроме вершины $0$ либо соединена с вершиной $0$ или соединена с вершиной, которая соединена с вершиной $0$.
  То есть для всех $i$ таких что $1 \le i \lt N$, или $P[i]=0$ или $P[P[i]]=0$.
1. (8 баллов) Для всех $c$ таких что $1 \le c \le M$ есть не более двух ребер цвета $c$.
1. (14 баллов) $N \le 200$ и $M \le 500$
1. (14 баллов) $N \le 2\,000$ и $M = 2$
1. (12 баллов) $N \le 2\,000$
1. (17 баллов) $M = 2$
1. (12 баллов) Без дополнительных ограничений.

## Sample Grader

Грейдер читает входные данные в следующем формате:

* строка $1$: $N \; M$
* строка $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* строка $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

Пусть $b[0], \; b[1], \; \ldots$ обозначает массив, который вернула функция `beechtree`.
Грейдер выводит ответ в единственной строке в следующем формате:
* строка $1$: $b[0] \; b[1] \; \ldots$
