# ブナの木 (Beech Tree)

Vétyem の森は色とりどりの木々が生えている森林として有名である． 
森の中でも最も古くて大きなブナの木のひとつには，Ős Vezér という名前がつけられている．

Ős Vezér と呼ばれる木は，$N$ 個の **頂点** と $N - 1$ 本の **辺** を持つグラフとして表現することができる．頂点には $0$ から $N - 1$ までの番号が，辺には $1$ から $N - 1$ までの番号が付けられている．それぞれの辺は異なる $2$ つの頂点の間を結んでいる．特に，辺 $i$ ($1 \le i \lt N$) は頂点 $i$ と頂点 $P[i]$ (ただし，$0 \le P[i] \lt i$) の間を結んでいる．頂点 $P[i]$ を頂点 $i$ の **親** と呼び，頂点 $i$ を頂点 $P[i]$ の **子** と呼ぶことにする．

それぞれの辺には色がついている．辺の色は，$1$ 以上 $M$ 以下の整数として表現される．辺 $i$ の色は $C[i]$ である．異なる辺が同じ色を持つこともある．

上の定義では，$i = 0$ は木のどの辺にも対応しないことに注意せよ．便宜上，$P[0] = -1$，$C[0] = 0$ であるとする．

例えば，Ős Vezér が $N = 18$ 頂点で，$M = 3$ 色の色で表現でき，$17$ 本の辺が $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ のように結ばれていて，色が $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$ であるとする．この木を図に表すと，以下のようになる．

![Tree](beechtree-tree.png "550")

Árpád は優れた森林学者で，**部分木** と呼ばれる木の特定の部分について研究することを好んでいる．$0 \le r \lt N$ なる各 $r$ について，頂点 $r$ の部分木とは，以下が成り立つような頂点の集合 $T(r)$ のことである．

* 頂点 $r$ は $T(r)$ に含まれる．
* 頂点 $x$ が $T(r)$ に含まれているならば，$x$ の子もすべて $T(r)$ に含まれる．
* これ以外の頂点は $T(r)$ に含まれない．

集合 $T(r)$ の大きさを $|T(r)|$ と書くことにする．

最近，Árpád は複雑であるが興味深い部分木の性質を発見した．
Árpád は紙と鉛筆を使ってたくさん遊んでこの性質を発見したので，あなたがこれを理解するには同じようにする必要があると思っている．
そこで，彼はあなたが詳細に解析できるように複数の例を示してくれる．

ある頂点 $r$ を選び，部分木 $T(r)$ に含まれる頂点たちのある順列 $v_0, v_1, \ldots, v_{|T(r)|-1}$ を選び，これを固定して考える．

$1 \le i \lt |T(r)|$ なる各 $i$ について，$f(i)$ を，長さ $i - 1$ の色の列 $C[v_1], C[v_2], \ldots, C[v_{i-1}]$ の中に色 $C[v_i]$ が出現する回数として定義する．

（$f(1)$ は必ず $0$ になることに注意せよ．これは，定義において色の列が空になるためである．）

順列 $v_0, v_1, \ldots, v_{|T(r)|-1}$ が **美しい順列** であるとは，順列が以下の条件を満たすことをいう．

* $v_0 = r$.
* $1 \le i \lt |T(r)|$ を満たす各 $i$ について，頂点 $v_i$ の親は頂点 $v_{f(i)}$ である．

$0 \le r \lt N$ なる各 $r$ について，部分木 $T(r)$ が **美しい部分木** であるとは，部分木が以下の条件を満たすことをいう．

* $T(r)$ の頂点の順列であって，美しい順列であるものが存在する．

定義により，$1$ 個の頂点からなる部分木は必ず美しい部分木になることに注意せよ．

上の例における木について考える．部分木 $T(0)$ と部分木 $T(3)$ は美しい部分木でないことが証明できる．部分木 $T(14)$ は $1$ 個の頂点からなる部分木であるため，美しい部分木である．以下に，部分木 $T(1)$ が美しい部分木であることを示す．

相異なる頂点の列 $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$ について考えよう．この列は $T(1)$ の頂点の順列である．この順列を図に表すと以下のようになる．頂点に付けられたラベルの番号は，この順列の何番目にその頂点が現れるかを表している．

![Tree](beechtree-label1.png "500")

以下で，この順列が *美しい順列* であることを確かめる．

* $v_0 = 1$ である.
* $C[v_1] = C[4] = 1$ は列 $[]$ に $0$ 回出現するため，$f(1) = 0$ である．
  * したがって，$v_1$ の親は $v_0$ でなければならない．確かに，頂点 $4$ の親は頂点 $1$ である．(形式的には，$P[4] = 1$ である．)
* $C[v_2] = C[5] = 2$ は列 $[1]$ に $0$ 回出現するため，$f(2) = 0$ である．
  * したがって，$v_2$ の親は $v_0$ でなければならない．確かに，頂点 $5$ の親は頂点 $1$ である．
* $C[v_3] = C[12] = 1$ は列 $[1, 2]$ に $1$ 回出現するため，$f(3) = 1$ である．
  * したがって，$v_3$ の親は $v_1$ でなければならない．確かに，頂点 $12$ の親は頂点 $4$ である．
* $C[v_4] = C[13] = 2$ は列 $[1, 2, 1]$ に $1$ 回出現するため，$f(4) = 1$ である．
  * したがって，$v_4$ の親は $v_1$ でなければならない．確かに，頂点 $13$ の親は頂点 $4$ である．
* $C[v_5] = C[6] = 3$ は列 $[1, 2, 1, 2]$ に $0$ 回出現するため，$f(5) = 0$ である．
  * したがって，$v_5$ の親は $v_0$ でなければならない．確かに，頂点 $6$ の親は頂点 $1$ である．
* $C[v_6] = C[14] = 2$ は列 $[1, 2, 1, 2, 3]$ に $2$ 回出現するため，$f(6) = 2$ である．
  * したがって，$v_6$ の親は $v_2$ でなければならない．確かに，頂点 $14$ の親は頂点 $5$ である．

$T(1)$ の頂点の順列であって *美しい順列* であるものが存在するので，部分木 $T(1)$ は *美しい部分木* である．

あなたの課題は，Ős Vezér のすべての部分木がそれぞれ美しいかどうかを判定して，Árpád を助けることである．

## 実装の詳細

以下の関数を実装せよ．

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: 木の頂点数．
* $M$: あり得る色の数．
* $P$, $C$: 木の辺の情報を表す長さ $N$ の配列．
* この関数は長さ $N$ の配列 $b$ を返さなければならない．$0 \le r \lt N$ なる各 $r$ について，$b[r]$ は $T(r)$ が美しい部分木ならば $1$ ，美しくないならば $0$ でなければならない．
* それぞれのテストケースについて，この関数はちょうど $1$ 回呼び出される．

## 例

### 例 $1$

以下の呼び出しを考える．

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

この木を図に表すと以下のようになる．

![Tree](beechtree-ex1.png "350")

$T(1), T(2), T(3)$ はそれぞれ $1$ 個の頂点からなる部分木であるため，美しい部分木である．$T(0)$ は美しい部分木ではない．したがって，この関数は $[0, 1, 1, 1]$ を返さなければならない．

### 例 $2$

以下の呼び出しを考える．

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

これは上の問題文で図示された例である．

この関数は $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$ を返さなければならない．

### 例 $3$

以下の呼び出しを考える．

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

この木を図に表すと以下のようになる．

![Tree](beechtree-ex3.png "400")

$T(0)$ が美しい部分木でない唯一の部分木である．この関数は $[0, 1, 1, 1, 1, 1, 1]$ を返さなければならない．


## 制約

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[i] \lt i$ ($1 \le i \lt N$)
* $1 \le C[i] \le M$ ($1 \le i \lt N$)
* $P[0] = -1$
* $C[0] = 0$

## 小課題

1. ($9$ 点) $N \le 8$ かつ $M \le 500$
1. ($5$ 点) 辺 $i$ は頂点 $i$ と頂点 $i-1$ の間を結ぶ．すなわち，$1 \le i \lt N$ なる各 $i$ について，$P[i] = i-1$ である．
1. ($9$ 点) 頂点 $0$ を除くすべての頂点は，頂点 $0$ と繋がっているか，もしくは，頂点 $0$ と繋がっている頂点と繋がっている．すなわち，$1 \le i \lt N$ なる各 $i$ について，$P[i]=0$ または $P[P[i]]=0$ が成り立つ．
1. ($8$ 点) $1 \le c \le M$ なる各 $c$ について，色が $c$ である辺は高々 $2$ 本である．
1. ($14$ 点) $N \le 200$ かつ $M \le 500$
1. ($14$ 点) $N \le 2\,000$ かつ $M = 2$
1. ($12$ 点) $N \le 2\,000$
1. ($17$ 点) $M = 2$
1. ($12$ 点) 追加の制約はない．

## 採点プログラムのサンプル

採点プログラムのサンプルは以下の形式で入力を読み込む．

* $1$ 行目: $N \; M$
* $2$ 行目: $P[0] \; P[1] \; \ldots \; P[N-1]$
* $3$ 行目: $C[0] \; C[1] \; \ldots \; C[N-1]$

関数 `beechtree` が返した配列の要素を $b[0], \; b[1], \; \ldots$ とする．採点プログラムのサンプルは以下の形式であなたの答えを出力する．

* $1$ 行目: $b[0] \; b[1] \; \ldots$
