# Beech Tree 櫸樹

Vétyem Woods 是一片有著許多色彩繽紛的樹木的著名林地。
其中一棵最古老且最高的櫸樹被稱為 Ős Vezér。

樹 Ős Vezér 可以被建模為一組 $N$ 個**節點**和 $N-1$ 個**邊**。
節點從 $0$ 到 $N-1$ 編號，邊從 $1$ 到 $N-1$ 編號。
每條邊連接樹的兩個不同節點。
具體而言，邊 $i$ ($1 \le i \lt N$) 將節點 $i$ 連接到節點 $P[i]$，其中 $0 \le P[i] \lt i$。節點 $P[i]$ 被稱為節點 $i$ 的**父節點**，節點 $i$ 被稱為節點 $P[i]$ 的**子節點**。

每條邊都有一種顏色。有 $M$ 種可能的邊顏色，從 $1$ 到 $M$ 編號。
邊 $i$ 的顏色是 $C[i]$。不同的邊可能有相同的顏色。

請注意，在上述定義中，情況 $i = 0$ 不對應於樹的一條邊。為了方便起見，我們讓 $P[0] = -1$ 和 $C[0] = 0$。

例如，假設 Ős Vezér 有 $N = 18$ 個節點和 $M = 3$ 種可能的邊顏色，其中 $17$ 條邊的連接為 $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ 和顏色 $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$ 描述。該樹顯示在下圖中：

![Tree](beechtree-tree.png "550")

Árpád 是一位很有才華的林務人員，喜歡研究樹的特定部分，稱為**子樹**。
對於每個 $r$，滿足 $0 \le r \lt N$，節點 $r$ 的子樹是具有以下特性的節點集合 $T(r)$：
* 節點 $r$ 屬於 $T(r)$。
* 每當節點 $x$ 屬於 $T(r)$，則 $x$ 的所有子節點也屬於 $T(r)$。
* 沒有其他節點屬於 $T(r)$。

集合 $T(r)$ 的大小表示為 $|T(r)|$。

Árpád 最近發現了一個複雜但有趣的子樹性質。Árpád 的發現涉及大量的筆記和紙張操作，他懷疑你可能也需要這樣做才能理解。他還會向你展示多個例子，供你進一步分析。

假設我們有一個固定的 $r$ 和一個節點子樹 $T(r)$ 中節點的排列 $v_0, v_1, \ldots, v_{|T(r)|-1}$。

對於每個 $i$，滿足 $1 \le i \lt |T(r)|$，令 $f(i)$ 為顏色 $C[v_i]$ 在以下 $i-1$ 個顏色的序列中出現的次數：$C[v_1], C[v_2], \ldots, C[v_{i-1}]$。

（請注意，$f(1)$ 總是 $0$，因為其定義中的顏色序列是空的。）

節點排列 $v_0, v_1, \ldots, v_{|T(r)|-1}$ 是一個**美麗的排列**，當且僅當所有以下性質都成立：
* $v_0 = r$。
* 對於每個 $i$，滿足 $1 \le i \lt |T(r)|$，節點 $v_i$ 的父節點是節點 $v_{f(i)}$。

對於任何滿足 $0 \le r \lt N$ 的 $r$，子樹 $T(r)$ 是一個**美麗的子樹**，當且僅當存在一個美麗的節點排列在 $T(r)$ 中的節點上。根據定義，每個只包含一個節點的子樹都是美麗的。

考慮上面的示例樹。
可以證明，該樹的子樹 $T(0)$ 和 $T(3)$ 不是美麗的。
子樹 $T(14)$ 是美麗的，因為它只包含一個節點。
下面，我們將展示子樹 $T(1)$ 也是美麗的。

考慮節點排列 $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$。這個排列是子樹 $T(1)$ 中節點的一個排列。下圖顯示了這個排列。附加在每個節點上的標籤就是這些節點在這個排列中的序號。

![Tree](beechtree-label1.png "500")

我們現在將驗證這是一個*美麗的排列*。

* $v_0 = 1$。
* $f(1) = 0$，因為顏色 $C[v_1] = C[4] = 1$ 在序列 $[]$ 中出現 $0$ 次。
  * 相應地，節點 $v_1$ 的父節點是節點 $v_0$。也就是說，節點 $4$ 的父節點是節點 $1$。（正式地說，$P[4] = 1$。）
* $f(2) = 0$，因為顏色 $C[v_2] = C[5] = 2$ 在序列 $[1]$ 中出現 $0$ 次。
  * 相應地，節點 $v_2$ 的父節點是節點 $v_0$。也就是說，節點 $5$ 的父節點是 $1$。
* $f(3) = 1$，因為顏色 $C[v_3] = C[12] = 1$ 在序列 $[1, 2]$ 中出現 $1$ 次。
  * 相應地，節點 $v_3$ 的父節點是節點 $v_1$。也就是說，節點 $12$ 的父節點是 $4$。
* $f(4) = 1$，因為顏色 $C[v_4] = C[13] = 2$ 在序列 $[1, 2, 1]$ 中出現 $1$ 次。
  * 相應地，節點 $v_4$ 的父節點是節點 $v_1$。也就是說，節點 $13$ 的父節點是 $4$。
* $f(5) = 0$，因為顏色 $C[v_5] = C[6] = 3$ 在序列 $[1, 2, 1, 2]$ 中出現 $0$ 次。
  * 相應地，節點 $v_5$ 的父節點是節點 $v_0$。也就是說，節點 $6$ 的父節點是 $1$。 
* $f(6) = 2$，因為顏色 $C[v_6] = C[14] = 2$ 在序列 $[1, 2, 1, 2, 3]$ 中出現 $2$ 次。
  * 相應地，節點$v_6$的父節點是$v_2$。 也就是說，節點$14$ 的父節點是 $5$。

由於我們可以在 $T(1)$ 中找到節點的“美麗的排列”，因此子樹 $T(1)$ 是一個“美麗的子樹”。

你的任務是幫助 Árpád 決定 Ős Vezér 的每個子樹是否美麗。


## 實現細節

你應該實現以下的函數。

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: 樹中節點的數量。
* $M$: 可能的邊的顏色數量。
* $P$, $C$: 長度為 $N$ 的陣列，描述樹的邊。
* 這個函數應該返回一個長度為 $N$ 的陣列 $b$。
  對於每個 $r$ 滿足 $0 \le r \lt N$，如果 $T(r)$ 是美麗的，$b[r]$ 應該是 $1$，否則為 $0$。
* 這個函數對於每個測試案例只會被調用一次。

## 範例

### 範例 1

考慮以下的調用：

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

樹如下圖所示：

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$ 和 $T(3)$ 各自只包含一個節點，因此它們是美麗的。
$T(0)$ 不是美麗的。
因此，函數應該返回 $[0, 1, 1, 1]$。

### 範例 2

考慮以下的調用：

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

這個範例在任務描述中有圖示。

函數應該返回 $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$。

### 範例 3

考慮以下的調用：

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

這個範例在下圖中有圖示。

![Tree](beechtree-ex3.png "400")

$T(0)$ 是唯一一個不是美麗的子樹。
函數應該返回 $[0, 1, 1, 1, 1, 1, 1]$。


## 約束

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[i] \lt i$（對於每個 $i$ 滿足 $1 \le i \lt N$）
* $1 \le C[i] \le M$（對於每個 $i$ 滿足 $1 \le i \lt N$）
* $P[0] = -1$ 且 $C[0] = 0$

## 子任務

1. （9 分）$N \le 8$ 且 $M \le 500$
1. （5 分）邊 $i$ 將節點 $i$ 連接到節點 $i-1$。也就是說，對於每個 $i$ 滿足 $1 \le i \lt N$，$P[i] = i-1$。
1. （9 分）除了節點 $0$ 之外的每個節點要麼連接到節點 $0$，要麼連接到一個連接到節點 $0$ 的節點。
  也就是說，對於每個 $i$ 滿足 $1 \le i \lt N$，要麼 $P[i]=0$，要麼 $P[P[i]]=0$。
1. （8 分）對於每個 $c$ 滿足 $1 \le c \le M$，顏色 $c$ 的邊最多只有兩條。
1. （14 分）$N \le 200$ 且 $M \le 500$
1. （14 分）$N \le 2\,000$ 且 $M = 2$
1. （12 分）$N \le 2\,000$
1. （17 分）$M = 2$
1. （12 分）沒有額外的約束。

## 範例測試程式

範例測試程式以以下格式讀取輸入：

* 第一行：$N \; M$
* 第二行：$P[0] \; P[1] \; \ldots \; P[N-1]$
* 第三行：$C[0] \; C[1] \; \ldots \; C[N-1]$

令 $b[0], \; b[1], \; \ldots$ 表示 `beechtree` 返回的陣列元素。
範例測試程式會用以下格式將你的答案輸出：

* 第一行：$b[0] \; b[1] \; \ldots$
