# Beech Tree

Biogradska gora je poznata prašuma s mnogo stabala.
Jedno od najstarijih stabala se zove Bukva.

Stablo Bukve se može predstaviti kao skup $N$ **čvorova** i $N-1$ **ivica** (edges).
Čvorovi su numerisani od $0$ do $N-1$ a ivice od $1$ do $N-1$.
Svaka ivica spaja dva različita čvora u stablu.
Tačnije, ivica $i$ ($1 \le i \lt N$) spaja čvor $i$ sa $P[i]$, gdje $0 \le P[i] \lt i$. Čvor $P[i]$ je **roditelj** (parent) od čvora $i$, a čvor $i$ je **dijete** čvora $P[i]$.

Svaka ivica ima boju. Postoji $M$ mogućih boja ivica, numerisanih od $1$ do $M$.
Boja čvora $i$ je $C[i]$.
Različite ivice mogu imati istu boju.

Primjetite da u definicijama iznad, slučaj $i = 0$ nije ivica drveta. Zbog jednostavnosti, pretpostavimo da je $P[0] = -1$ i $C[0] = 0$.

Na primjer, pretpostavimo da Bukva ima $N = 18$ čvorova i $M = 3$ mogućih boja, sa $17$ ivica opisanih vezama $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ i bojama $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$. 
Stablo je prikazano slici ispod:

![Tree](beechtree-tree.png "550")

Marko je talentovan šumar koji voli da proučava specifične dijelove stabla zvane **podstabla**.
Za svako $r$ gdje važi $0 \le r \lt N$, podstablo čvora $r$ je skup čvorova $T(r)$ sa sljedećim osobinama:
* čvor $r$ pripada $T(r)$.
* Kada god čvor $x$ pripada $T(r)$, sva djeca od $x$ takođe pripadaju $T(r)$.
* Ni jedan drugi čvor ne pripada $T(r)$.

Broj članova skupa $T(r)$ se označava sa $|T(r)|$.

Marko je nedavno otkrio komplikovano, ali zanimljivo svojstvo podstabla. Markovo otkriće uključivalo je dosta igranja s olovkom i papirom i on misli da biste i vi trebali učiniti isto da biste ga razumjeli. Pokazaće vam više primjera koje zatim možete detaljno analizirati.

Pretpostavimo da imamo fiksirano $r$ i permutaciju $v_0, v_1, \ldots, v_{|T(r)|-1}$ čvorova u podstablu $T(r)$.

Za svako $i$ takvo da $1 \le i \lt |T(r)|$, neka je $f(i)$ broj koliko se puta boja $C[v_i]$ pojavljuje u sljedećem nizu $i-1$ boja: $C[v_1], C[v_2], \ldots, C[v_{i-1}]$.

(Primjetite da $f(1)$ je uvijek $0$ jer niz boja je u toj definiciji prazan.)

**Permutacija** $v_0, v_1, \ldots, v_{|T(r)|-1}$ **je lijepa** ako i samo ako sljedeće osobine važe:
* $v_0 = r$.
* Za svako $i$ takvo da $1 \le i \lt |T(r)|$, roditelj čvora $v_i$ je čvor $v_{f(i)}$.

Za bilo koje $r$ takvo da $0 \le r \lt N$, podstablo $T(r)$ je **lijepo podstablo** ako i samo ako postoji lijepa permutacija čvorova u $T(r)$. Svako podstablo koje sadrži samo jedan čvor je *lijepo*.

Uzmimo za primjer stablo iznad.
Može se vidjeti da podstabla $T(0)$ i $T(3)$ nisu *lijepa*.
Podstablo $T(14)$ je *lijepo*, jer ima samo jedan čvor.
Ispod, pokazaćemo da je podstablo $T(1)$ takođe *lijepo*.

Uzmimo skup brojeva $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$.
Ovaj niz je permutacija čvorova u $T(1)$.
Slika ispod predstavlja ovu permutaciju.
Index svakog čvora u ovoj permutaciji je prikazan kao broj u "zastavici" pored čvora.

![Tree](beechtree-label1.png "500")

Očito, niz iznad je permutacija čvorova u $T(1)$. A sada ćemo provjeriti da li je permutacija zaista *lijepa*.

* $v_0 = 1$.
* $f(1) = 0$ pošto $C[v_1] = C[4] = 1$ se pojavljuje $0$ puta u nizu $[]$.
* Samim tim, roditelj $v_1$ je $v_0$. (tačnije roditelj $4$ je $1$. - Odnosno $P[4] = 1$.)
* $f(2) = 0$ pošto $C[v_2] = C[5] = 2$ se pojavljuje $0$ puta u nizu $[1]$.
* Samim tim, roditelj $v_2$ je $v_0$. (tačnije roditelj $5$ je $1$.)
* $f(3) = 1$ pošto $C[v_3] = C[12] = 1$ se pojavljuje $1$ put u nizu $[1, 2]$.
* Samim tim, roditelj $v_3$ je $v_1$. (tačnije roditelj $12$ je $4$.)
* $f(4) = 1$ pošto $C[v_4] = C[13] = 2$ se pojavljuje $1$ put u nizu $[1, 2, 1]$.
* Samim tim, roditelj $v_4$ je $v_1$. (tačnije roditelj $13$ je $4$.)
* $f(5) = 0$ pošto $C[v_5] = C[6] = 3$ se pojavljuje $0$ puta u nizu $[1, 2, 1, 2]$.
* Samim tim, roditelj $v_5$ je $v_0$. (tačnije roditelj $6$ je $1$.)
* $f(6) = 2$ pošto $C[v_6] = C[14] = 2$ se pojavljuje $2$ puta u nizu $[1, 2, 1, 2, 3]$.
* Samim tim, roditelj $v_6$ je $v_2$. (tačnije roditelj $14$ je $5$.)

Pošto smo mogli pronaći *lijepu permutaciju* čvorova u $T(1)$, podstablo $T(1)$ je zaista *lijepo podstablo*.

Vaš zadatak je da pomognete Marku da odluči za svako podstablo Bukve da li je *lijepo*, jer izgleda da ipak nije toliko talentovan šumar koliko je mislio. Možda je trebao upisati IT akademiju.

## Detalji implementacije

Trebate implementirati sljedeću funkciju.

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: je broj čvorova u stablu.
* $M$: je broj mogućih boja u stablu.
* $P$, $C$: su nizovi dužine $N$ koji opisuju ivice stabla.
* Ova funkcija bi trebala da vrati niz $b$ dužine $N$. 
  Za svako $r$ takvo da $0 \le r \lt N$, $b[r]$ treba biti $1$ ako je $T(r)$ *lijepo*, ili $0$ u suprotnom.
* Ova se funkcija poziva tačno jednom za svaki testni slučaj.

## Primjeri

### Primjer 1

Uzmimo za primjer sljedeći poziv:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

Stablo je prikazano ispod:

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$, i $T(3)$ svako ima samo jedan čvor i samim tim $T(0)$ nije *lijepo*.
Tako da, funkcija treba da vrati $[0, 1, 1, 1]$.

### Primjer 2

Uzmimo u obzir poziv ispod:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

Ovaj primjer je prikazan u tekstu zadatka.

Funkcija bi trebala vratiti $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

### Primjer 3

Uzmimo u obzir sljedeći poziv:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

Primjer je prikazan u slici ispod.

![Tree](beechtree-ex3.png "400")

$T(0)$ je jedino podstablo koje nije *lijepo*.
Funkcija bi trebala da vrati $[0, 1, 1, 1, 1, 1, 1]$.


## Ograničenja

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[i] \lt i$ (za svako $i$ takvo da $1 \le i \lt N$)
* $1 \le C[i] \le M$ (za svako $i$ takvo da $1 \le i \lt N$)
* $P[0] = -1$ i $C[0] = 0$

## Podzadaci

1. (9 bodova) $N \le 8$ and $M \le 500$
1. (5 bodova) Ivica $i$ spaja čvor $i$ sa čvorom $i-1$. Tačnije, za svako $i$ takvo da $1 \le i \lt N$, $P[i] = i-1$.
1. (9 bodova) Svaki čvor osim čvora $0$ je ili spojen sa čvorom $0$, ili je spojen sa čvorom koji je spojen sa $0$.
  Odnosno, za svako $i$ takvo da $1 \le i \lt N$, je ili $P[i]=0$ ili $P[P[i]]=0$.
1. (8 bodova) Za svako $c$ takvo da $1 \le c \le M$, postoje najviše dva čvora sa bojom $c$.
1. (14 bodova) $N \le 200$ i $M \le 500$
1. (14 bodova) $N \le 2\,000$ i $M = 2$
1. (12 bodova) $N \le 2\,000$
1. (17 bodova) $M = 2$
1. (12 bodova) Bez dodatnih ograničenja.

## Sample Grader

Sample grader očekuje ulaz u sljedećem formatu:

* linija $1$: $N \; M$
* linija $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* linija $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

Neka $b[0], \; b[1], \; \ldots$ predstavljaju elemente niza vraćenih iz funkcije `beechtree`.
Sample grader ispisuje vaš odgovor u jednoj liniji u sljedećem formatu:
* line $1$: $b[0] \; b[1] \; \ldots$
