# Árbol Fagus

El Bosque de Vétyem es famoso por su variedad de especies de árboles coloridos.
Uno de los más antiguos y grandes, de la especie Fagus, se llama Ős Vezér.


Ős Vezér puede ser representado como un conjunto de $N$ **nodos** y $N-1$ **aristas**.
Los nodos están numerados del $0$ al $N-1$ y las aristas están numeradas del $1$ al $N-1$.
Cada arista conecta dos nodos distintos del árbol.
Específicamente, la arista $v$ ($1 \le v \lt N$) conecta el nodo $v$ con el nodo $P[v]$, donde $0 \le P[v] \lt v$.

Cada arista tiene un color. Hay $M$ posibles colores de aristas, numerados del $1$ al $M$.
El color de la arista $v$ es $C[v]$.
Distintas aristas pueden tener el mismo color.

Nótese que en las definición anterior, el caso $v = 0$ no corresponde a una arista del árbol. Por conveniencia, decimos que $P[0] = -1$ y $C[0] = 0$.

Por ejemplo, supongamos que Ős Vezér tiene $N = 18$ nodos y $M = 3$ posibles colores de aristas, con 17 aristas descritas por las conexiones  $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ y colores  $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$. 

El árbol se muestra a continuación:


![Tree](beechtree-tree.png "550")

Árpád es un talentoso guardabosques al que le gusta estudiar partes específicas del árbol llamadas **subárboles**.
Para cada $r$ tal que $0 \le r \lt N$, el subárbol del nodo $r$ (denotado como $T(r)$) es un conjunto de nodos.
Un nodo $s$ es un miembro de  $T(r)$ si y solo si:
* $s$ = $r$, o
* $s \gt r$ y el nodo $P[s]$ pertenece a $T(r)$.

El tamaño del conjunto $T(r)$ se denota por $|T(r)|$.

Árpád descubrió una propiedad interesante para los subárboles.
Para cada $r$ tal que $0 \le r \lt N$, el subárbol $T(r)$ es **enrevesado** si y solo si existe una secuencia de enteros *distintos* $[v_0, v_1, \ldots, v_{|T(r)|-1}]$ tal que:

* Para cada $i$ tal que $0 \le i \lt |T(r)|$, el nodo $v_i$ pertenece a $T(r)$.
* $v_0 = r$.
* Para cada $i$ tal que $1 \le i \lt |T(r)|$, $P[v_i] = v_{f(i)}$, donde $f(i)$ se define como la cantidad de veces que el color $C[v_i]$ aparece en la secuencia $[C[v_1], C[v_2], \ldots, C[v_{i-1}]]$.

Nótese que acorde a la definición:

* Cada subárbol que contiene un único nodo es entrevesado.
* Para cada súbarbol que contiene dos o más nodos,  $f(1) = 0$ porque la secuencia de colores en su definición es vacía.


Considera el ejemplo del árbol de arriba.
Los subárboles $T(0)$ y $T(3)$ de este árbol no son enrevesados.
El subárbol $T(14)$ es enrevesado, ya que contiene un único nodo.
A continuación demostraremos que el subárbol $T(1)$ también es enrevesado.

Considera la secuencia de enteros distintos $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$.

Esta secuencia está representada en la siguiente figura.
El índice de cada nodo en la secuencia se muestra como el número de la etiqueta de cada nodo.

![Tree](beechtree-label1.png "500")

La secuencia de enteros mostrada demuestra que $T(1)$ es enrevesado:
* $v_0 = r = 1$.
* $f(1) = 0$ ya que $C[v_1] = C[4] = 1$ aparece $0$ veces en la secuencia $[]$, y $P[v_1] = P[4] = 1 = v_0$.
* $f(2) = 0$ ya que $C[v_2] = C[5] = 2$ aparece $0$ veces en la secuencia $[1]$, y $P[v_2] = P[5] = 1 = v_0$.
* $f(3) = 1$ ya que $C[v_3] = C[12] = 1$ aparece $1$ vez en la secuencia $[1, 2]$, y $P[v_3] = P[12] = 4 = v_1$.
* $f(4) = 1$ ya que $C[v_4] = C[13] = 2$ aparece $1$ vez en la secuencia $[1, 2, 1]$, y $P[v_4] = P[13] = 4 = v_1$.
* $f(5) = 0$ ya que $C[v_5] = C[6] = 3$ aparece $0$ veces en la secuencia $[1, 2, 1, 2]$, y $P[v_5] = P[6] = 1 = v_0$.
* $f(6) = 2$ ya que $C[v_6] = C[14] = 2$ aparece $2$ veces en la secuencia $[1, 2, 1, 2, 3]$, y $P[v_6] = P[14] = 5 = v_2$.

Tu tarea es ayudar a Árpád a decidir si cada subárbol de Ős Vezér es enrevesado o no.

## Detalles de Implementación

Debes implementar el siguiente método.

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: la cantidad de nodos del árbol.
* $M$: la cantidad de posibles colores de aristas.
* $P$, $C$: arreglos de tamaño $N$ que describen las aristas del árbol.
* El método debe regresar un arreglo $b$ de tamaño $N$. 
  Para cada $r$ tal que $0 \le r \lt N$, $b[r]$ debe ser $1$ si $T(r)$ es enrevesado, y $0$ en caso contrario.
* Este método se llama exactamente una vez para cada caso de prueba.

## Ejemplos

### Ejemplo 1

Considera la siguiente llamada:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

El árbol se muestra en la siguiente figura:

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$, y $T(3)$ contienen cada uno un único nodo y por lo tanto son enrevesados.
$T(0)$ no es enrevesado.
Por lo tanto, el método debe regresar $[0, 1, 1, 1]$.

### Ejemplo 2

Considera la siguiente llamada:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

Este ejemplo se muestra en la descripción del problema más arriba.

El método debe regresar $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

### Ejemplo 3

Considera la siguiente llamada:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

Este ejemplo se muestra en la siguiente figura.

![Tree](beechtree-ex3.png "400")

$T(0)$ es el único subárbol que no es enrevesado.
El método debe regresar $[0, 1, 1, 1, 1, 1, 1]$.


## Restricciones

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[v] \lt v$ (para cada $v$ tal que $1 \le v \lt N$)
* $1 \le C[v] \le M$ (para cada $v$ tal que $1 \le v \lt N$)
* $P[0] = -1$ y $C[0] = 0$

## Subtareas

1. (9 puntos) $N \le 8$ y $M \le 500$
2. (5 puntos) La arista $v$ conecta al nodo $v$ al nodo $v-1$. Esto es, para cada $v$ tal que $1 \le v \lt N$, $P[v] = v-1$.
3. (9 puntos) Cada nodo distinto al nodo $0$ está conectado al nodo $0$, o está conectado a un nodo que está conectado al nodo $0$.
 Esto es, para cada $v$ tal que $1 \le v \lt N$, se cumple que $P[v]=0$ o $P[P[v]]=0$, pero no ambas.
4. (8 puntos) Para cada $c$ tal que $1 \le c \le M$, hay a lo mucho dos aristas de color $c$.
5. (14 puntos) $N \le 200$ y $M \le 500$
6. (14 puntos) $N \le 2\,000$ y $M = 2$
7. (12 puntos) $N \le 2\,000$
8. (17 puntos) $M = 2$
9. (12 puntos) Sin restricciones adicionales.

## Evaluador Local

El evaluador local lee la entrada en el siguiente formato:

* línea $1$: $N \; M$
* línea $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* línea $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

Sean $b[0], \; b[1], \; \ldots$ los elementos regresados por `beechtree`.
El evaluador local imprime tu respuesta en una única línea, en el siguiente formato:
* línea $1$: $b[0] \; b[1] \; \ldots$
