# Árbol de Haya

Vétyem Woods es un famoso bosque con muchos árboles coloridos.
Uno de los árboles de haya más viejos y altos se llama Ős Vezér.

El árbol Ős Vezér puede ser modelado como un conjunto de $N$ **nodos** y $N-1$ **aristas**.
Los nodos son numerados del $0$ al $N-1$ y las aristas son numeradas de la $1$ a la $N-1$.
Cada arista conecta dos nodos distintos del árbol.
Específicamente, la arista $i$ ($1 \le i \lt N$) conecta al nodo $i$ con el nodo $P[i]$, donde $0 \le P[i] \lt i$. El nodo $P[i]$ es llamado el **padre** del nodo $i$, y el nodo $i$ es llamado un **hijo** del nodo $P[i]$.

Cada arista tiene un color. Hay $M$ posibles colores de arista numerados del $1$ al $M$.
El color de la arista $i$ es $C[i]$.
Diferentes aristas pueden tener el mismo color.

Observa que, en la definición anterior, el caso $i = 0$ no corresponde a una arista del árbol. Por conveniencia, se asignan $P[0] = -1$ y $C[0] = 0$.

Por ejemplo, supongamos que Ős Vezér tiene $N = 18$ nodos y $M = 3$ posibles colores para las aristas, con $17$ aristas descritas por las conexiones $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ y los colores $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$. El árbol se muestra en la siguiente figura:

![Tree](beechtree-tree.png "550")

Árpád es un guardabosques talentoso que le gusta estudiar partes específicas del árbol llamadas **subárboles**.
Para cada $r$ tal que $0 \le r \lt N$, el subárbol del nodo $r$ es el conjunto $T(r)$ de nodos con las siguientes propiedades:
* Nodo $r$ pertenece a $T(r)$.
* Cuando un nodo $x$ pertenece a $T(r)$, todos los hijos de $x$ también pertenecen a $T(r)$.
* Ningún otro nodo pertenece a $T(r)$.

El tamaño del conjunto $T(r)$ se denota como $|T(r)|$.

Árpád descubrió recientemente una propiedad complicada pero interesante de los subárboles. El descubrimiento de Árpád necesitó utilizar mucho papel y lápiz, y él sospecha que tendrás que hacer lo mismo para entenderlo. También te mostrará múltiples ejemplos para que los analices cuidadosamente.

Supongamos que tenemos un $r$ fijo y una permutación $v_0, v_1, \ldots, v_{|T(r)|-1}$ de los nodos del subárbol $T(r)$.

Para cada $i$ tal que $1 \le i \lt |T(r)|$, sea $f(i)$ el número de veces que el color $C[v_i]$ aparece en la siguiente secuencia de $i-1$ colores: $C[v_1], C[v_2], \ldots, C[v_{i-1}]$.

Note que $f(1)$ siempre es $0$ ya que la secuencia de colores en su definición es vacía.

La permutación $v_0, v_1, \ldots, v_{|T(r)|-1}$ es una **permutación hermosa** si y solo si todas las propiedades siguientes se cumplen:
* $v_0 = r$.
* Para cada $i$ tal que $1 \le i \lt |T(r)|$, el padre del nodo $v_i$ es el nodo $v_{f(i)}$.

Para cualquier $r$ tal que $0 \le r \lt N$, el subárbol $T(r)$ es un **subárbol hermoso** si y solo si existe una permutación hermosa de los nodos en $T(r)$. Note que, de acuerdo a esta definición, cada subárbol que contenga un único nodo es hermoso.

Considera el árbol de ejemplo anterior.
Puede demostrarse que los subárboles $T(0)$ y $T(3)$  no son hermosos.
El subárbol $T(14)$ es hermoso, ya que es formado por un único nodo.
Abajo se mostrará que el subárbol $T(1)$ también es hermoso.

Considera la secuencia de enteros distintos $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$.
Esta secuencia es una permutación de los nodos en $T(1)$. La figura siguiente muestra esta permutación. Las etiquetas en los nodos son los índices en los que dichos nodos aparecen en la permutación.

![Tree](beechtree-label1.png "500")

Ahora verificaremos que esta es una *permutación hermosa*:

* $v_0 = r = 1$.
* $f(1) = 0$ ya que $C[v_1] = C[4] = 1$ aparece $0$ veces en la secuencia $[]$.
 * Correspondientemente, el padre de $v_1$ es $v_0$. Es decir, el padre del nodo $4$ es el nodo $1$. Formalmente, $P[4] = 1$.
* $f(2) = 0$ ya que $C[v_2] = C[5] = 2$ aparece $0$ veces en la secuencia $[1]$.
 * Correspondientemente, el padre de $v_2$ es $v_0$. Es decir, el padre de $5$ es $1$.
* $f(3) = 1$ ya que $C[v_3] = C[12] = 1$ aparece $1$ vez en la secuencia $[1, 2]$.
 * Correspondientemente, el padre de $v_3$ es $v_1$. Es decir, el padre de $12$ es $4$.
* $f(4) = 1$ ya que $C[v_4] = C[13] = 2$ aparece $1$ vez en la secuencia $[1, 2, 1]$.
 * Correspondientemente, el padre de $v_4$ es $v_1$. Es decir, el padre de $13$ es $4$.
* $f(5) = 0$ ya que $C[v_5] = C[6] = 3$ aparece $0$ veces en la secuencia $[1, 2, 1, 2]$.
 * Correspondientemente, el padre de $v_5$ es $v_0$. Es decir, el padre de $6$ es $1$.
* $f(6) = 2$ ya que $C[v_6] = C[14] = 2$ aparece $2$ veces en la secuencia $[1, 2, 1, 2, 3]$.
 * Correspondientemente, el padre de $v_6$ es $v_2$. Es decir, el padre de $14$ es $5$.

Como logramos encontrar una *permutación hermosa* de los nodos en $T(1)$, el subárbol $T(1)$ es un *subárbol hermoso*.

Tu tarea es ayudar a Árpád a decidir, para cada subárbol de Ős Vezér, si es hermoso.

## Detalles de implementación

Debes implementar la siguiente función.

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: el número de nodos en el árbol.
* $M$: el número de posibles colores para las aristas.
* $P$, $C$: arreglos de longitud $N$ que describen las aristas del árbol.
* Esta función debe devolver un arreglo $b$ de longitud $N$. 
  Para cada $r$ tal que $0 \le r \lt N$, $b[r]$ debería ser $1$ si $T(r)$ es hermoso, y $0$ en caso contrario.
* Esta función es llamada exactamente una vez para cada caso de prueba.

## Ejemplos

### Ejemplo 1

Considera la siguiente llamada:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

El árbol se muestra en la siguiente figura:

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$, y $T(3)$ están formados por un único nodo cada uno, por lo que son hermosos.
$T(0)$ no es hermoso.
Por lo tanto, la función debe retornar $[0, 1, 1, 1]$.

### Ejemplo 2

Considera la siguiente llamada:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

Este ejemplo se ilustra en la descripción del problema.

La función debe retornar $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

### Ejemplo 3

Considera la siguiente llamada:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

Este ejemplo se ilustra en la siguiente figura.

![Tree](beechtree-ex3.png "400")

$T(0)$ es el único subárbol que no es hermoso.
La función debe retornar $[0, 1, 1, 1, 1, 1, 1]$.


## Restricciones

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[i] \lt v$ (para cada $i$ tal que $1 \le i \lt N$)
* $1 \le C[i] \le M$ (para cada $i$ tal que $1 \le i \lt N$)
* $P[0] = -1$ y $C[0] = 0$

## Subtareas 

1. (9 puntos) $N \le 8$ y $M \le 500$
1. (5 puntos) La arista $i$ conecta el nodo $i$ con el nodo $i-1$. Es decir, para cada $i$ tal que $1 \le i \lt N$, $P[i] = i-1$.
1. (9 puntos) Cada nodo diferente del nodo $0$ o está conectado al nodo $0$, o está conectado a un nodo que está conectado al nodo $0$.
  Es decir, para cada $i$ tal que $1 \le i \lt N$, o bien $P[i]=0$ o $P[P[i]]=0$.
1. (8 puntos) Para cada $c$ tal que $1 \le c \le M$, hay como máximo dos aristas de color $c$.
1. (14 puntos) $N \le 200$ y $M \le 500$
1. (14 puntos) $N \le 2\,000$ y $M = 2$
1. (12 puntos) $N \le 2\,000$
1. (17 puntos) $M = 2$
1. (12 puntos) Sin restricciones adicionales.

## Evaluador local

El evaluador local lee la entrada en el siguiente formato:

* línea $1$: $N \; M$
* línea $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* línea $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

Sean $b[0], \; b[1], \; \ldots$ los elementos del arreglo devuelto por `beechtree`.
El evaluador local imprime tu respuesta en una única línea, en el siguiente formato:
* línea $1$: $b[0] \; b[1] \; \ldots$