# Буква

Шумадија је позната регија у којој је некада било много дрвећа.
Једна од најстаријих букви у Шумадији се зове *Бабауква*.

Стабло *Бабауква* се може представити као скуп $N$ **чворова** и $N-1$ **грана**.
Чворови су означени од $0$ до $N-1$, а гране од $1$ до $N-1$.
Свака грана спаја два различита чвора у стаблу.
Тачније, грана $v$ ($1 \le v \lt N$) спаја чвор $v$ са $P[v]$, где је $0 \le P[v] \lt v$. Чвор $P[i]$ се зове **родитељ** чвора $i$, и чвор $i$ се зове **дете** чвора $P[i]$.

Свака грана има боју. Постоји $M$ могућих боја грана, означених од $1$ до $M$.
Боја чвора $v$ је $C[v]$.
Различите гране могу имати исту боју.

Приметити да у претходним дефиницијама, случај $v = 0$ не представља грану стабла. Зарад једноставности, претпоставимо да је $P[0] = -1$ и $C[0] = 0$.

На пример, претпоставимо да *Бабауква* има $N = 18$ чворова и $M = 3$ могућих боја, са $17$ грана описаних везама $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$ и бојама $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$. Стабло је приказано на следећој слици:

![Tree](beechtree-tree.png "550")

Дракче је талентован шумар који воли да проучава поједине делове стабала зване **подстабла**.
За свако $r$ за које важи $0 \le r \lt N$, подстабло чвора $r$ (означено са $T(r)$) је скуп чворова.
Чвор $s$ је члан $T(r)$ ако и само ако важи:
* $s$ = $r$, или
* $s \gt r$ и чвор $P[s]$ је члан $T(r)$.

Величина скупа $T(r)$ се означава са $|T(r)|$.

Дракче је открио једну занимљиву особину подстабла.
За свако $r$ такво да $0 \le r \lt N$, подстабло $T(r)$ је **компликовано** ако и само ако постоји секвенца *јединствених* бројева $[v_0, v_1, \ldots, v_{|T(r)|-1}]$ (пермутација чворова) таквих да:
* За свако $i$ такво да $0 \le i \lt |T(r)|$, чвор $v_i$ је члан $T(r)$.
* $v_0 = r$.
* За свако $i$ такво да $1 \le i \lt |T(r)|$, $P[v_i] = v_{f(i)}$, где је $f(i)$ дефинисано као број колико се пута боја $C[v_i]$ појављује у низу $[C[v_1], C[v_2], \ldots, C[v_{i-1}]]$.

Приметити да по дефиницији:
* Свако подстабло које садржи само један чвор је *компликовано*.
* За свако подстабло које има два или више чворова, $f(1) = 0$ јер је низ боја у том случају празан.

Узмимо за пример стабло изнад.
Подстабла $T(0)$ и $T(3)$ нису *компликована*.
Подстабло $T(14)$ је *компликовано*, јер садржи само један чвор.
У наставку ће бити показано да је подстабло $T(1)$ такође *компликовано*.

Размотримо скуп различитих целих бројева $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$.
Овај низ је представљен на нааредној слици.
Индекс сваког чвора у овом скупу је приказан као број у "заставици" поред чвора.

![Tree](beechtree-label1.png "500")

Низ бројева приказаних изнад показује да је $T(1)$ *компликовано*:
* $v_0 = r = 1$.
* $f(1) = 0$ јер $C[v_1] = C[4] = 1$ се појављује $0$ пута у низу $[]$, и $P[v_1] = P[4] = 1 = v_0$.
* $f(2) = 0$ јер $C[v_2] = C[5] = 2$ се појављује $0$ пута у низу $[1]$, и $P[v_2] = P[5] = 1 = v_0$.
* $f(3) = 1$ јер $C[v_3] = C[12] = 1$ се појављује $1$ пут у низу $[1, 2]$, и $P[v_3] = P[12] = 4 = v_1$.
* $f(4) = 1$ јер $C[v_4] = C[13] = 2$ се појављује $1$ пут у низу $[1, 2, 1]$, и $P[v_4] = P[13] = 4 = v_1$.
* $f(5) = 0$ јер $C[v_5] = C[6] = 3$ се појављује $0$ пута у низу $[1, 2, 1, 2]$, и $P[v_5] = P[6] = 1 = v_0$.
* $f(6) = 2$ јер $C[v_6] = C[14] = 2$ се појављује $2$ пута у низу $[1, 2, 1, 2, 3]$, и $P[v_6] = P[14] = 5 = v_2$.

Ваш задатак је да помогнете Дракчету да одлучи за свако подстабло *Бабаукве* да ли је *компликовано*.

## Појединости имплементације

Потребно је имплементирати следећу функцију.

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: је број чворова у стаблу.
* $M$: је број могућих боја у стаблу.
* $P$, $C$: су низови дужине $N$ који описују ивице стабла.
* Ова функција би требала да враћа низ $b$ дужине $N$.
  За свако $r$ такво да $0 \le r \lt N$, $b[r]$ треба да буде $1$ ако је $T(r)$ *компликовано*, или $0$ у супротном.
* Ова се функција позива тачно једном за сваки тестни случај.

## Примери

### Пример 1

Узмимо за пример следећи позив:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

Стабло је приказано на следећој слици:

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$, и $T(3)$ свако садржи само један чвор па су самим тим *компликована*.
$T(0)$ није *компликовано*.
Тако да, функција треба да врати $[0, 1, 1, 1]$.

### Пример 2

Размотримо следећи позив:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

Овај пример је приказан у поставци задатка.

Функција би требала да врати $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$.

### Пример 3

Узмимо у разматрање следећи позив:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

Пример је илустрован на следећој слици.

![Tree](beechtree-ex3.png "400")

$T(0)$ је једино подстабло које није *компликовано*.
Функција би требала да врати $[0, 1, 1, 1, 1, 1, 1]$.


## Ограничења

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[v] \lt v$ (за свако $v$ такво да $1 \le v \lt N$)
* $1 \le C[v] \le M$ (за свако $v$ такво да $1 \le v \lt N$)
* $P[0] = -1$ и $C[0] = 0$

## Подзадаци

1. (9 бодова) $N \le 8$ и $M \le 500$
1. (5 бодова) Грана $v$ спаја чвор $v$ са чвором $v-1$. Тачније, за свако $v$ такво да $1 \le v \lt N$, $P[v] = v-1$.
1. (9 бодова) Сваки чвор осим чвора $0$ је или спојен са чвором $0$, или је спојен са чвором који је спојен са чвором $0$. Односно, за свако $v$ такво да $1 \le v \lt N$, је или $P[v]=0$ или $P[P[v]]=0$.
1. (8 бодова) За свако $c$ такво да $1 \le c \le M$, постоје највише два чвора са бојом $c$.
1. (14 бодова) $N \le 200$ и $M \le 500$
1. (14 бодова) $N \le 2\,000$ и $M = 2$
1. (12 бодова) $N \le 2\,000$
1. (17 бодова) $M = 2$
1. (12 бодова) Без додатних ограничења.

## Тестни оцењивач

Тестни оцењивач учитава улаз у следећем формату:

* линија $1$: $N \; M$
* линија $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* линија $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

Нека $b[0], \; b[1], \; \ldots$ означавају чланове низа враћених из функције `beechtree`.
Тестни оцењивач исписује ваш одговор у једној линији у следећем формату::
* линија $1$: $b[0] \; b[1] \; \ldots$
