# 山毛櫸樹

Vétyem Woods 是一個有名的林地，樹林中有很多彩色的樹。其中最老及最高的山毛櫸樹叫做Ős Vezér。

Ős Vezér 可以視為有 $N$ 個 **節點** 以及 $N-1$ 個 **邊** 的集合。節點的編號由 $0$ 編到 $N-1$， 邊的編號 由 $1$ 編到 $N-1$。每個邊連結樹中的兩個相異節點。更明確地，編號為 $i$ ($1 \le i \lt N$) 的邊連接節點 $i$ 到節點 $P[i]$，其中 $0 \le P[i] \lt i$。節點 $P[i]$ 稱為節點 $i$ 的 **父節點** ，且節點 $i$ 被稱為節點 $P[i]$ 的 **子節點**。

每一個邊有一顏色。邊的顏色總共有 $M$ 種可能， 其編號由$1$ 到 $M$。編號為 $i$ 的邊，其顏色為 $C[i]$。不同的邊可以有相同的顏色。

注意在上述的定義中，當 $i = 0$ 時，樹中沒有對應的邊。為了方便起見，我們令 $P[0] = -1$ 及 $C[0] = 0$。

例如，假設 Ős Vezér 有 $N = 18$ 個節點及 $M = 3$ 種可能的顏色，並包含 $17$ 個邊，其連接方式如: $P = [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11]$，其顏色分布如: $C = [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3]$。此樹如下圖所示:

![Tree](beechtree-tree.png "550")

Árpád 是一位有才華的林務員，喜歡研究樹的特定部分，稱為 **子樹** 。 對每一個 $r$ $(0 \le r \lt N)$，節點 $r$ 的子樹為$T(r)$ 是一個具有下列性質的節點集合:
* 節點 $r$ 屬於 $T(r)$。
* 當節點 $x$ 屬於 $T(r)$，所有 $x$ 的子節點都屬於 $T(r)$。
* $T(r)$ 沒有其它成員。

集合 $T(r)$ 的大小以 $|T(r)|$ 表示。

Árpád 最近發現了一複雜但有趣的子樹性質。Árpád 的發現過程用了許多紙筆的嘗試，而且他猜想你也需要經歷過類似的過程以了解這性質。他也將給你幾個可以仔細分析的例子。

<!--對每一個 $r$ $(0 \le r \lt N)$， 子樹 $T(r)$ 為 ** 漂亮** 若且唯若存在 $T(r)$ 的節點的*漂亮排列*。-->

假設我們有一固定的 $r$ 和子樹 $T(r)$ ，其節點排列為  $v_0, v_1, \ldots, v_{|T(r)|-1}$ 。 

對每一個  $i$ $(1 \le i \lt |T(r)|)$，令 $f(i)$ 為顏色 $C[v_i]$ 出現在下列 $i-1$個顏色的序列 $C[v_1], C[v_2], \ldots, C[v_{i-1}]$  的次數。

(注意 $f(1)$ 的值總是 $0$，因為按其定義顏色序列為空集合。)

排列 $v_0, v_1, \ldots, v_{|T(r)|-1}$ 被稱為 **漂亮排列** 若且唯若所有下列性質都成立:
* $v_0 = r$ 。
* 對每一 $i$  $(1 \le i \lt |T(r)|)$,  $v_i$ 的父節點為節點 $v_{f(i)}$。

對任一 $r$ $(0\le r \le N)$，子樹 $T(r )$ 是一個 **漂亮子樹** 若且唯若 $T(r )$ 的節點存在一漂亮排列。注意按照定義，任一只有一個節點的子樹是漂亮的。

考慮上述的例子。可以驗證子樹 $T(0)$ 和 $T(3)$ 都不是漂亮的。子樹 $T(14)$ 為漂亮的，因為它只有一個節點。下面我們將解釋子樹 $T(1)$ 也是漂亮的。

考慮相異整數序列 $[v_0, v_1, v_2, v_3, v_4, v_5, v_6] = [1, 4, 5, 12, 13, 6, 14]$。此序列為 $T(1)$ 的節點的排列。此排列如下圖所示。每一節點在排列中的索引值 (index)為附貼在節點旁的數字。

![Tree](beechtree-label1.png "500")

現在我們將驗證它是一個 *漂亮排列* 。

* $v_0  = 1$.
* $f(1) = 0$， 因為在序列 $[]$中，$C[v_1] = C[4] = 1$ 出現 $0$ 次。
 * 相對應地，$v_1$ 的父節點為 $v_0$，即 4 的父節點是 1，(正式地， $P[4]=1$)。
* $f(2) = 0$， 因為在序列 $[1]$ 中， $C[v_2] = C[5] = 2$ 出現 $0$ 次。
 * 相對應地，$v_2$ 的父節點為 $v_0$，即 5 的父節點是 1。
* $f(3) = 1$，因為在序列 $[1, 2]$ 中， $C[v_3] = C[12] = 1$ 出現 $1$ 次。
 * 相對應地，$v_3$ 的父節點為 $v_1$，即 12 的父節點是 4。
* $f(4) = 1$，因為在序列 $[1, 2, 1]$ 中， $C[v_4] = C[13] = 2$ 出現 $1$ 次。
 * 相對應地，$v_4$ 的父節點為 $v_1$，即 13 的父節點是 4。
* $f(5) = 0$，因為在序列 $[1, 2, 1, 2]$ 中， $C[v_5] = C[6] = 3$ 出現 $0$次。
 * 相對應地，$v_5$ 的父節點為 $v_0$，即 6 的父節點是 1。
* $f(6) = 2$ ，因為在序列 $[1, 2, 1, 2, 3]$ 中， $C[v_6] = C[14] = 2$ 出現 $2$ 次。
 * 相對應地，$v_6$ 的父節點為 $v_2$，即 14 的父節點是 5。

因為我們可以找到 $T(1)$ 的節點的 *漂亮排列* ， 子樹 $T(1)$ 是一個 *漂亮子樹*。

你的任務是幫 Árpád 判定 Ős Vezér 的每一個子樹是否為漂亮的。

## 實作細節

你應實作下列程序。

```
int[] beechtree(int N, int M, int[] P, int[] C)
```

* $N$: 樹的節點數。
* $M$: 邊的可能顏色個數。
* $P$, $C$: 分別是長度為 $N$ 的陣列，用以描述邊的資訊。
* 此程序應回傳一長度為 $N$ 的陣列 $b$。對每一  $r$  $(0 \le r \lt N)$，若 $T(r)$ 為 漂亮則 $b[r]$ 應為 $1$，否則為 $0$。
* 對每一筆測資，此程序恰好被呼叫一次。

## 範例

### 範例 1

考慮下列呼叫:

```
beechtree(4, 2, [-1, 0, 0, 0], [0, 1, 1, 2])
```

這棵樹如下圖所示:

![Tree](beechtree-ex1.png "350")

$T(1)$, $T(2)$ 和 $T(3)$ 分別都只包含一個節點，因此為漂亮的。
$T(0)$ 則不是漂亮的。因之，程序應回傳 $[0, 1, 1, 1]$。

### 範例 2

考慮下列呼叫:

```
beechtree(18, 3, 
          [-1, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 10, 11, 11],
          [0, 1, 2, 3, 1, 2, 3, 1, 3, 3, 2, 1, 1, 2, 2, 1, 2, 3])
```

本範例，如之前問題敘述中所示。

程序應回傳 $[0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]$。

### 範例 3

考慮下列呼叫:

```
beechtree(7, 2, [-1, 0, 1, 1, 0, 4, 5], [0, 1, 1, 2, 2, 1, 1])
```

本範例如下圖所示。

![Tree](beechtree-ex3.png "400")

$T(0)$ 是唯一不是漂亮 的子樹。程序應回傳 $[0, 1, 1, 1, 1, 1, 1]$。


## 限制

* $3 \le N \le 200\,000$
* $2 \le M \le 200\,000$
* $0 \le P[i] \lt i$ ($i$ 的範圍為 $1 \le i \lt N$)
* $1 \le C[i] \le M$ ($i$ 的範圍為 $1 \le i \lt N$)
* $P[0] = -1$且 $C[0] = 0$

## 子任務

1. (9 points) $N \le 8$ 且 $M \le 500$
1. (5 points) 編號為 $i$ 的邊連結節點 $i$ 到節點 $i-1$。也就是，對每一個 $i$  $(1 \le i \lt N)$, $P[i] = i-1$
1. (9 points) 除了節點 $0$ 之外，每一個節點連接到節點 $0$ 或連接到一個連到節點 $0$ 的節點。也就是，對每一 $i$ $(1 \le i \lt N)$， $P[i]=0$ 或 $P[P[i]]=0$，但不會同時成立。
1. (8 points) 對每一 $c$ $(1 \le c \le M) $，最多有兩個邊的顏色為 $c$。
1. (14 points) $N \le 200$ 且 $M \le 500$
1. (14 points) $N \le 2\,000$ 且 $M = 2$
1. (12 points) $N \le 2\,000$
1. (17 points) $M = 2$
1. (12 points) 無其他限制。

## 評分程式

樣本評分程式以下列格式讀取輸入:

* line $1$: $N \; M$
* line $2$: $P[0] \; P[1] \; \ldots \; P[N-1]$
* line $3$: $C[0] \; C[1] \; \ldots \; C[N-1]$

令 $b[0], \; b[1], \; \ldots$ 表示 `beechtree` 所回傳的陣列元素。
樣本評分程式以下列格式輸出答案:
* line $1$: $b[0] \; b[1] \; \ldots$
